@@ -6,15 +6,22 @@
 */
 
 #ifndef lint
-static char dkim_canon_c_id[] = "@(#)$Id: dkim-canon.c,v 1.19 2010/06/03 15:07:55 cm-msk Exp $";
+static char dkim_canon_c_id[] = "@(#)$Id: dkim-canon.c,v 1.21.10.1 2010/10/27 21:43:08 cm-msk Exp $";
 #endif /* !lint */
 
 #include "build-config.h"
 
+/* for Solaris */
+#ifndef _REENTRANT
+# define _REENTRANT
+#endif /* _REENTRANT */
+
 /* system includes */
 #include <sys/param.h>
 #include <sys/types.h>
-#include <stdbool.h>
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#endif /* HAVE_STDBOOL_H */
 #include <assert.h>
 #include <string.h>
 #include <ctype.h>
@@ -44,8 +51,8 @@ static char dkim_canon_c_id[] = "@(#)$Id: dkim-canon.c,v 1.19 2010/06/03 15:07:5
 #include "util.h"
 
 /* definitions */
-#define	CRLF	"\r\n"
-#define	SP	" "
+#define	CRLF	(u_char *) "\r\n"
+#define	SP	(u_char *) " "
 
 /* macros */
 #define	DKIM_ISLWSP(x)	((x) == 011 || (x) == 013 || (x) == 014 || (x) == 040)
@@ -76,6 +83,32 @@ dkim_canon_free(DKIM *dkim, DKIM_CANON *canon)
 	{
 		switch (canon->canon_hashtype)
 		{
+#ifdef USE_GNUTLS
+		  case DKIM_HASHTYPE_SHA1:
+		  case DKIM_HASHTYPE_SHA256:
+		  {
+			struct dkim_sha *sha;
+
+			sha = (struct dkim_sha *) canon->canon_hash;
+
+			if (sha->sha_tmpfd != -1)
+			{
+				close(sha->sha_tmpfd);
+				sha->sha_tmpfd = -1;
+			}
+
+			gnutls_hash_deinit(sha->sha_hd, NULL);
+
+			if (sha->sha_out != NULL)
+			{
+				DKIM_FREE(dkim, sha->sha_out);
+				sha->sha_out = NULL;
+			}
+
+			break;
+		  }
+
+#else /* USE_GNUTLS */
 		  case DKIM_HASHTYPE_SHA1:
 		  {
 			struct dkim_sha1 *sha1;
@@ -92,7 +125,7 @@ dkim_canon_free(DKIM *dkim, DKIM_CANON *canon)
 			break;
 		  }
 
-#ifdef SHA256_DIGEST_LENGTH
+# ifdef HAVE_SHA256
 		  case DKIM_HASHTYPE_SHA256:
 		  {
 			struct dkim_sha256 *sha256;
@@ -108,7 +141,8 @@ dkim_canon_free(DKIM *dkim, DKIM_CANON *canon)
 
 			break;
 		  }
-#endif /* SHA256_DIGEST_LENGTH */
+# endif /* HAVE_SHA256 */
+#endif /* USE_GNUTLS */
 
 		  default:
 			assert(0);
@@ -158,6 +192,22 @@ dkim_canon_write(DKIM_CANON *canon, u_char *buf, size_t buflen)
 
 	switch (canon->canon_hashtype)
 	{
+#ifdef USE_GNUTLS
+	  case DKIM_HASHTYPE_SHA1:
+	  case DKIM_HASHTYPE_SHA256:
+	  {
+		struct dkim_sha *sha;
+
+		sha = (struct dkim_sha *) canon->canon_hash;
+
+		gnutls_hash(sha->sha_hd, buf, buflen);
+
+		if (sha->sha_tmpfd != -1)
+			(void) write(sha->sha_tmpfd, buf, buflen);
+
+		break;
+	  }
+#else /* USE_GNUTLS */
 	  case DKIM_HASHTYPE_SHA1:
 	  {
 		struct dkim_sha1 *sha1;
@@ -171,7 +221,7 @@ dkim_canon_write(DKIM_CANON *canon, u_char *buf, size_t buflen)
 		break;
 	  }
 
-#ifdef SHA256_DIGEST_LENGTH
+# ifdef HAVE_SHA256
 	  case DKIM_HASHTYPE_SHA256:
 	  {
 		struct dkim_sha256 *sha256;
@@ -184,7 +234,8 @@ dkim_canon_write(DKIM_CANON *canon, u_char *buf, size_t buflen)
 
 		break;
 	  }
-#endif /*SHA256_DIGEST_LENGTH */
+# endif /* HAVE_SHA256 */
+#endif /* USE_GNUTLS */
 	}
 
 	canon->canon_wrote += buflen;
@@ -247,21 +298,22 @@ dkim_canon_buffer(DKIM_CANON *canon, u_char *buf, size_t buflen)
 }
 
 /*
-**  DKIM_CANON_HEADER -- canonicalize a header and write it
+**  DKIM_CANON_HEADER_STRING -- canonicalize a header field
 **
 **  Parameters:
-**  	dkim -- DKIM handle
-**  	canon -- DKIM_CANON handle
-**  	hdr -- header handle
+**  	dstr -- dkim_dstring to use for output
+**  	canon -- canonicalization mode to apply
+**  	hdr -- header field input
+**  	hdrlen -- bytes to process at "hdr"
 **  	crlf -- write a CRLF at the end?
 **
 **  Return value:
 **  	A DKIM_STAT constant.
 */
 
-static DKIM_STAT
-dkim_canon_header(DKIM *dkim, DKIM_CANON *canon, struct dkim_header *hdr,
-                  _Bool crlf)
+DKIM_STAT
+dkim_canon_header_string(struct dkim_dstring *dstr, dkim_canon_t canon,
+                         unsigned char *hdr, size_t hdrlen, _Bool crlf)
 {
 	_Bool space;
 	int n;
@@ -270,40 +322,25 @@ dkim_canon_header(DKIM *dkim, DKIM_CANON *canon, struct dkim_header *hdr,
 	u_char *end;
 	u_char tmpbuf[BUFRSZ];
 
-	assert(canon != NULL);
+	assert(dstr != NULL);
 	assert(hdr != NULL);
 
 	tmp = tmpbuf;
 	end = tmpbuf + sizeof tmpbuf - 1;
 
-	if (dkim->dkim_canonbuf == NULL)
-	{
-		dkim->dkim_canonbuf = dkim_dstring_new(dkim, hdr->hdr_textlen,
-		                                       0);
-		if (dkim->dkim_canonbuf == NULL)
-			return DKIM_STAT_NORESOURCE;
-	}
-	else
-	{
-		dkim_dstring_blank(dkim->dkim_canonbuf);
-	}
-
 	n = 0;
 
-	dkim_canon_buffer(canon, NULL, 0);
-
-	switch (canon->canon_canon)
+	switch (canon)
 	{
 	  case DKIM_CANON_SIMPLE:
-		if (!dkim_dstring_catn(dkim->dkim_canonbuf,
-		                       hdr->hdr_text, hdr->hdr_textlen) ||
-		    (crlf && !dkim_dstring_catn(dkim->dkim_canonbuf, CRLF, 2)))
+		if (!dkim_dstring_catn(dstr, hdr, hdrlen) ||
+		    (crlf && !dkim_dstring_catn(dstr, CRLF, 2)))
 			return DKIM_STAT_NORESOURCE;
 		break;
 
 	  case DKIM_CANON_RELAXED:
 		/* process header field name (before colon) first */
-		for (p = hdr->hdr_text; *p != '\0'; p++)
+		for (p = hdr; p < hdr + hdrlen; p++)
 		{
 			/*
 			**  Discard spaces before the colon or before the end
@@ -332,7 +369,7 @@ dkim_canon_header(DKIM *dkim, DKIM_CANON *canon, struct dkim_header *hdr,
 			{
 				*tmp = '\0';
 
-				if (!dkim_dstring_catn(dkim->dkim_canonbuf,
+				if (!dkim_dstring_catn(dstr,
 				                       tmpbuf, tmp - tmpbuf))
 					return DKIM_STAT_NORESOURCE;
 
@@ -376,7 +413,7 @@ dkim_canon_header(DKIM *dkim, DKIM_CANON *canon, struct dkim_header *hdr,
 				{
 					*tmp = '\0';
 
-					if (!dkim_dstring_catn(dkim->dkim_canonbuf,
+					if (!dkim_dstring_catn(dstr,
 					                       tmpbuf,
 					                       tmp - tmpbuf))
 						return DKIM_STAT_NORESOURCE;
@@ -395,7 +432,7 @@ dkim_canon_header(DKIM *dkim, DKIM_CANON *canon, struct dkim_header *hdr,
 			{
 				*tmp = '\0';
 
-				if (!dkim_dstring_catn(dkim->dkim_canonbuf,
+				if (!dkim_dstring_catn(dstr,
 				                       tmpbuf, tmp - tmpbuf))
 					return DKIM_STAT_NORESOURCE;
 
@@ -408,17 +445,64 @@ dkim_canon_header(DKIM *dkim, DKIM_CANON *canon, struct dkim_header *hdr,
 		{
 			*tmp = '\0';
 
-			if (!dkim_dstring_catn(dkim->dkim_canonbuf,
+			if (!dkim_dstring_catn(dstr,
 			                       tmpbuf, tmp - tmpbuf))
 				return DKIM_STAT_NORESOURCE;
 		}
 
-		if (crlf && !dkim_dstring_catn(dkim->dkim_canonbuf, CRLF, 2))
+		if (crlf && !dkim_dstring_catn(dstr, CRLF, 2))
 			return DKIM_STAT_NORESOURCE;
 
 		break;
 	}
 
+	return DKIM_STAT_OK;
+}
+
+/*
+**  DKIM_CANON_HEADER -- canonicalize a header and write it
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	canon -- DKIM_CANON handle
+**  	hdr -- header handle
+**  	crlf -- write a CRLF at the end?
+**
+**  Return value:
+**  	A DKIM_STAT constant.
+*/
+
+static DKIM_STAT
+dkim_canon_header(DKIM *dkim, DKIM_CANON *canon, struct dkim_header *hdr,
+                  _Bool crlf)
+{
+	DKIM_STAT status;
+
+	assert(dkim != NULL);
+	assert(canon != NULL);
+	assert(hdr != NULL);
+
+	if (dkim->dkim_canonbuf == NULL)
+	{
+		dkim->dkim_canonbuf = dkim_dstring_new(dkim, hdr->hdr_textlen,
+		                                       0);
+		if (dkim->dkim_canonbuf == NULL)
+			return DKIM_STAT_NORESOURCE;
+	}
+	else
+	{
+		dkim_dstring_blank(dkim->dkim_canonbuf);
+	}
+
+	dkim_canon_buffer(canon, NULL, 0);
+
+	status = dkim_canon_header_string(dkim->dkim_canonbuf,
+	                                  canon->canon_canon,
+	                                  hdr->hdr_text, hdr->hdr_textlen,
+	                                  crlf);
+	if (status != DKIM_STAT_OK)
+		return status;
+
 	dkim_canon_buffer(canon, dkim_dstring_get(dkim->dkim_canonbuf),
 	                  dkim_dstring_len(dkim->dkim_canonbuf));
 
@@ -447,178 +531,6 @@ dkim_canon_flushblanks(DKIM_CANON *canon)
 	canon->canon_blanks = 0;
 }
 
-/*
-**  DKIM_CANON_SELECTHDRS -- choose headers to be included in canonicalization
-**
-**  Parameters:
-**  	dkim -- DKIM context in which this is performed
-**  	hdrlist -- string containing headers that should be marked, separated
-**  	           by the ":" character
-**  	ptrs -- array of header pointers (modified)
-**  	nptr -- number of pointers available at "ptrs"
-**
-**  Return value:
-**  	Count of headers added to "ptrs", or -1 on error.
-**
-**  Notes:
-**  	Selects headers to be passed to canonicalization and the order in
-**  	which this is done.  "ptrs" is populated by pointers to headers
-**  	in the order in which they should be fed to canonicalization.
-**
-**  	If any of the returned pointers is NULL, then a header named by
-**  	"hdrlist" was not found.
-*/
-
-static int
-dkim_canon_selecthdrs(DKIM *dkim, u_char *hdrlist, struct dkim_header **ptrs,
-                      int nptrs)
-{
-	int c;
-	int n;
-	int m;
-	int shcnt;
-	size_t len;
-	u_char *bar;
-	char *ctx;
-	char *colon;
-	struct dkim_header *hdr;
-	struct dkim_header **lhdrs;
-	u_char **hdrs;
-
-	assert(dkim != NULL);
-	assert(ptrs != NULL);
-	assert(nptrs != 0);
-
-	/* if there are no headers named, use them all */
-	if (hdrlist == NULL)
-	{
-		n = 0;
-
-		for (hdr = dkim->dkim_hhead; hdr != NULL; hdr = hdr->hdr_next)
-		{
-			if (n >= nptrs)
-			{
-				dkim_error(dkim, "too many headers (max %d)",
-				           nptrs);
-				return -1;
-			}
-			ptrs[n] = hdr;
-			n++;
-		}
-
-		return n;
-	}
-
-	if (dkim->dkim_hdrlist == NULL)
-	{
-		dkim->dkim_hdrlist = dkim_malloc(dkim->dkim_libhandle,
-		                                 dkim->dkim_closure,
-		                                 DKIM_MAXHEADER);
-		if (dkim->dkim_hdrlist == NULL)
-		{
-			dkim_error(dkim, "unable to allocate %d bytes(s)",
-			           DKIM_MAXHEADER);
-
-			return -1;
-		}
-	}
-
-	strlcpy(dkim->dkim_hdrlist, hdrlist, DKIM_MAXHEADER);
-
-	/* mark all headers as not used */
-	for (hdr = dkim->dkim_hhead; hdr != NULL; hdr = hdr->hdr_next)
-		hdr->hdr_flags &= ~DKIM_HDR_SIGNED;
-
-	n = dkim->dkim_hdrcnt * sizeof(struct dkim_header *);
-	lhdrs = DKIM_MALLOC(dkim, n);
-	if (lhdrs == NULL)
-		return -1;
-	memset(lhdrs, '\0', n);
-
-	shcnt = 1;
-	for (colon = dkim->dkim_hdrlist; *colon != '\0'; colon++)
-	{
-		if (*colon == ':')
-			shcnt++;
-	}
-	n = sizeof(u_char *) * shcnt;
-	hdrs = DKIM_MALLOC(dkim, n);
-	if (hdrs == NULL)
-	{
-		(void) DKIM_FREE(dkim, lhdrs);
-		return -1;
-	}
-	memset(hdrs, '\0', n);
-
-	n = 0;
-
-	/* make a split-out copy of hdrlist */
-	for (bar = strtok_r(dkim->dkim_hdrlist, ":", &ctx);
-	     bar != NULL;
-	     bar = strtok_r(NULL, ":", &ctx))
-	{
-		hdrs[n] = (u_char *) bar;
-		n++;
-	}
-
-	/* for each named header, find the last unused one and use it up */
-	shcnt = 0;
-	for (c = 0; c < n; c++)
-	{
-		lhdrs[shcnt] = NULL;
-
-		len = MIN(DKIM_MAXHEADER, strlen(hdrs[c]));
-		while (len > 0 &&
-		       isascii(hdrs[c][len - 1]) &&
-		       isspace(hdrs[c][len - 1]))
-			len--;
-
-		for (hdr = dkim->dkim_hhead; hdr != NULL; hdr = hdr->hdr_next)
-		{
-			if (hdr->hdr_flags & DKIM_HDR_SIGNED)
-				continue;
-
-			if (len == hdr->hdr_namelen &&
-			    strncasecmp(hdr->hdr_text, hdrs[c], len) == 0)
-				lhdrs[shcnt] = hdr;
-		}
-
-		if (lhdrs[shcnt] != NULL)
-		{
-			lhdrs[shcnt]->hdr_flags |= DKIM_HDR_SIGNED;
-			shcnt++;
-		}
-	}
-
-	/* bounds check */
-	if (shcnt > nptrs)
-	{
-		dkim_error(dkim, "too many headers (found %d, max %d)", shcnt,
-		           nptrs);
-
-		DKIM_FREE(dkim, lhdrs);
-		DKIM_FREE(dkim, hdrs);
-
-		return -1;
-	}
-
-	/* copy to the caller's buffers */
-	m = 0;
-	for (c = 0; c < shcnt; c++)
-	{
-		if (lhdrs[c] != NULL)
-		{
-			ptrs[m] = lhdrs[c];
-			m++;
-		}
-	}
-
-	DKIM_FREE(dkim, lhdrs);
-	DKIM_FREE(dkim, hdrs);
-
-	return m;
-}
-
 /*
 **  DKIM_CANON_FIXCRLF -- rebuffer a body chunk, fixing "naked" CRs and LFs
 **
@@ -689,7 +601,6 @@ dkim_canon_fixcrlf(DKIM *dkim, DKIM_CANON *canon, u_char *buf, size_t buflen)
 	return DKIM_STAT_OK;
 }
 
-
 /* ========================= PUBLIC SECTION ========================= */
 
 /*
@@ -718,9 +629,8 @@ dkim_canon_init(DKIM *dkim, _Bool tmp, _Bool keep)
 		cur->canon_hashbuf = DKIM_MALLOC(dkim, DKIM_HASHBUFSIZE);
 		if (cur->canon_hashbuf == NULL)
 		{
-			dkim_error(dkim,
-			           "unable to allocate %d byte(s)",
-			           sizeof(struct dkim_sha1));
+			dkim_error(dkim, "unable to allocate %d byte(s)",
+			           DKIM_HASHBUFSIZE);
 			return DKIM_STAT_NORESOURCE;
 		}
 		cur->canon_hashbufsize = DKIM_HASHBUFSIZE;
@@ -731,6 +641,60 @@ dkim_canon_init(DKIM *dkim, _Bool tmp, _Bool keep)
 
 		switch (cur->canon_hashtype)
 		{
+#ifdef USE_GNUTLS
+		  case DKIM_HASHTYPE_SHA1:
+		  case DKIM_HASHTYPE_SHA256:
+		  {
+			struct dkim_sha *sha;
+
+			sha = (struct dkim_sha *) DKIM_MALLOC(dkim,
+			                                      sizeof(struct dkim_sha));
+			if (sha == NULL)
+			{
+				dkim_error(dkim,
+				           "unable to allocate %d byte(s)",
+				           sizeof(struct dkim_sha));
+				return DKIM_STAT_NORESOURCE;
+			}
+
+			memset(sha, '\0', sizeof(struct dkim_sha));
+			sha->sha_tmpfd = -1;
+
+			/* XXX -- test for errors */
+			if (cur->canon_hashtype == DKIM_HASHTYPE_SHA1)
+			{
+				(void) gnutls_hash_init(&sha->sha_hd,
+				                        GNUTLS_DIG_SHA1);
+			}
+			else
+			{
+				(void) gnutls_hash_init(&sha->sha_hd,
+				                        GNUTLS_DIG_SHA256);
+			}
+
+			if (sha->sha_hd == NULL)
+			{
+				DKIM_FREE(dkim, sha);
+				return DKIM_STAT_INTERNAL;
+			}
+				
+			if (tmp)
+			{
+				status = dkim_tmpfile(dkim, &fd, keep);
+				if (status != DKIM_STAT_OK)
+				{
+					DKIM_FREE(dkim, sha);
+					return status;
+				}
+
+				sha->sha_tmpfd = fd;
+			}
+
+			cur->canon_hash = sha;
+
+		  	break;
+		  }
+#else /* USE_GNUTLS */
 		  case DKIM_HASHTYPE_SHA1:
 		  {
 			struct dkim_sha1 *sha1;
@@ -766,7 +730,7 @@ dkim_canon_init(DKIM *dkim, _Bool tmp, _Bool keep)
 		  	break;
 		  }
 
-#ifdef SHA256_DIGEST_LENGTH
+# ifdef HAVE_SHA256
 		  case DKIM_HASHTYPE_SHA256:
 		  {
 			struct dkim_sha256 *sha256;
@@ -801,7 +765,8 @@ dkim_canon_init(DKIM *dkim, _Bool tmp, _Bool keep)
 
 		  	break;
 		  }
-#endif /* SHA256_DIGEST_LENGTH */
+# endif /* HAVE_SHA256 */
+#endif /* USE_GNUTLS */
 
 		  default:
 			assert(0);
@@ -880,12 +845,15 @@ dkim_add_canon(DKIM *dkim, _Bool hdr, dkim_canon_t canon, int hashtype,
 
 	assert(dkim != NULL);
 	assert(canon == DKIM_CANON_SIMPLE || canon == DKIM_CANON_RELAXED);
-#ifdef DKIM_HASHTYPE_SHA256
-	assert(hashtype == DKIM_HASHTYPE_SHA1 ||
-	       hashtype == DKIM_HASHTYPE_SHA256);
-#else /* DKIM_HASHTYPE_SHA256 */
-	assert(hashtype == DKIM_HASHTYPE_SHA1);
-#endif /* DKIM_HASHTYPE_SHA256 */
+	if (dkim_libfeature(dkim->dkim_libhandle, DKIM_FEATURE_SHA256))
+	{
+		assert(hashtype == DKIM_HASHTYPE_SHA1 ||
+		       hashtype == DKIM_HASHTYPE_SHA256);
+	}
+	else
+	{
+		assert(hashtype == DKIM_HASHTYPE_SHA1);
+	}
 
 	if (!hdr)
 	{
@@ -961,6 +929,179 @@ dkim_add_canon(DKIM *dkim, _Bool hdr, dkim_canon_t canon, int hashtype,
 	return DKIM_STAT_OK;
 }
 
+/*
+**  DKIM_CANON_SELECTHDRS -- choose headers to be included in canonicalization
+**
+**  Parameters:
+**  	dkim -- DKIM context in which this is performed
+**  	hdrlist -- string containing headers that should be marked, separated
+**  	           by the ":" character
+**  	ptrs -- array of header pointers (modified)
+**  	nptr -- number of pointers available at "ptrs"
+**
+**  Return value:
+**  	Count of headers added to "ptrs", or -1 on error.
+**
+**  Notes:
+**  	Selects headers to be passed to canonicalization and the order in
+**  	which this is done.  "ptrs" is populated by pointers to headers
+**  	in the order in which they should be fed to canonicalization.
+**
+**  	If any of the returned pointers is NULL, then a header named by
+**  	"hdrlist" was not found.
+*/
+
+int
+dkim_canon_selecthdrs(DKIM *dkim, u_char *hdrlist, struct dkim_header **ptrs,
+                      int nptrs)
+{
+	int c;
+	int n;
+	int m;
+	int shcnt;
+	size_t len;
+	char *bar;
+	char *ctx;
+	u_char *colon;
+	struct dkim_header *hdr;
+	struct dkim_header **lhdrs;
+	u_char **hdrs;
+
+	assert(dkim != NULL);
+	assert(ptrs != NULL);
+	assert(nptrs != 0);
+
+	/* if there are no headers named, use them all */
+	if (hdrlist == NULL)
+	{
+		n = 0;
+
+		for (hdr = dkim->dkim_hhead; hdr != NULL; hdr = hdr->hdr_next)
+		{
+			if (n >= nptrs)
+			{
+				dkim_error(dkim, "too many headers (max %d)",
+				           nptrs);
+				return -1;
+			}
+			ptrs[n] = hdr;
+			n++;
+		}
+
+		return n;
+	}
+
+	if (dkim->dkim_hdrlist == NULL)
+	{
+		dkim->dkim_hdrlist = dkim_malloc(dkim->dkim_libhandle,
+		                                 dkim->dkim_closure,
+		                                 DKIM_MAXHEADER);
+		if (dkim->dkim_hdrlist == NULL)
+		{
+			dkim_error(dkim, "unable to allocate %d bytes(s)",
+			           DKIM_MAXHEADER);
+
+			return -1;
+		}
+	}
+
+	strlcpy((char *) dkim->dkim_hdrlist, (char *) hdrlist, DKIM_MAXHEADER);
+
+	/* mark all headers as not used */
+	for (hdr = dkim->dkim_hhead; hdr != NULL; hdr = hdr->hdr_next)
+		hdr->hdr_flags &= ~DKIM_HDR_SIGNED;
+
+	n = dkim->dkim_hdrcnt * sizeof(struct dkim_header *);
+	lhdrs = DKIM_MALLOC(dkim, n);
+	if (lhdrs == NULL)
+		return -1;
+	memset(lhdrs, '\0', n);
+
+	shcnt = 1;
+	for (colon = dkim->dkim_hdrlist; *colon != '\0'; colon++)
+	{
+		if (*colon == ':')
+			shcnt++;
+	}
+	n = sizeof(u_char *) * shcnt;
+	hdrs = DKIM_MALLOC(dkim, n);
+	if (hdrs == NULL)
+	{
+		(void) DKIM_FREE(dkim, lhdrs);
+		return -1;
+	}
+	memset(hdrs, '\0', n);
+
+	n = 0;
+
+	/* make a split-out copy of hdrlist */
+	for (bar = strtok_r((char *) dkim->dkim_hdrlist, ":", &ctx);
+	     bar != NULL;
+	     bar = strtok_r(NULL, ":", &ctx))
+	{
+		hdrs[n] = (u_char *) bar;
+		n++;
+	}
+
+	/* for each named header, find the last unused one and use it up */
+	shcnt = 0;
+	for (c = 0; c < n; c++)
+	{
+		lhdrs[shcnt] = NULL;
+
+		len = MIN(DKIM_MAXHEADER, strlen((char *) hdrs[c]));
+		while (len > 0 &&
+		       isascii(hdrs[c][len - 1]) &&
+		       isspace(hdrs[c][len - 1]))
+			len--;
+
+		for (hdr = dkim->dkim_hhead; hdr != NULL; hdr = hdr->hdr_next)
+		{
+			if (hdr->hdr_flags & DKIM_HDR_SIGNED)
+				continue;
+
+			if (len == hdr->hdr_namelen &&
+			    strncasecmp((char *) hdr->hdr_text,
+			                (char *) hdrs[c], len) == 0)
+				lhdrs[shcnt] = hdr;
+		}
+
+		if (lhdrs[shcnt] != NULL)
+		{
+			lhdrs[shcnt]->hdr_flags |= DKIM_HDR_SIGNED;
+			shcnt++;
+		}
+	}
+
+	/* bounds check */
+	if (shcnt > nptrs)
+	{
+		dkim_error(dkim, "too many headers (found %d, max %d)", shcnt,
+		           nptrs);
+
+		DKIM_FREE(dkim, lhdrs);
+		DKIM_FREE(dkim, hdrs);
+
+		return -1;
+	}
+
+	/* copy to the caller's buffers */
+	m = 0;
+	for (c = 0; c < shcnt; c++)
+	{
+		if (lhdrs[c] != NULL)
+		{
+			ptrs[m] = lhdrs[c];
+			m++;
+		}
+	}
+
+	DKIM_FREE(dkim, lhdrs);
+	DKIM_FREE(dkim, hdrs);
+
+	return m;
+}
+
 /*
 **  DKIM_CANON_RUNHEADERS -- run the headers through all header
 **                           canonicalizations
@@ -1099,7 +1240,8 @@ dkim_canon_runheaders(DKIM *dkim)
 				/* terminate the header field name and test */
 				hdr->hdr_text[hdr->hdr_namelen] = '\0';
 				status = regexec(&lib->dkiml_hdrre,
-				                 hdr->hdr_text, 0, NULL, 0);
+				                 (char *) hdr->hdr_text,
+				                 0, NULL, 0);
 
 				/* restore the character */
 				hdr->hdr_text[hdr->hdr_namelen] = savechar;
@@ -1239,6 +1381,36 @@ dkim_canon_runheaders(DKIM *dkim)
 		/* finalize */
 		switch (cur->canon_hashtype)
 		{
+#ifdef USE_GNUTLS
+		  case DKIM_HASHTYPE_SHA1:
+		  case DKIM_HASHTYPE_SHA256:
+		  {
+			int alg;
+			struct dkim_sha *sha;
+
+			sha = (struct dkim_sha *) cur->canon_hash;
+
+			if (cur->canon_hashtype == DKIM_HASHTYPE_SHA1)
+				alg = GNUTLS_DIG_SHA1;
+			else
+				alg = GNUTLS_DIG_SHA256;
+
+			sha->sha_outlen = gnutls_hash_get_len(alg);
+
+			sha->sha_out = DKIM_MALLOC(dkim, sha->sha_outlen);
+			if (sha->sha_out == NULL)
+			{
+				dkim_error(dkim, "unable to allocate %u bytes",
+				           sha->sha_outlen);
+				return DKIM_STAT_NORESOURCE;
+			}
+
+			gnutls_hash_output(sha->sha_hd, sha->sha_out);
+
+			break;
+		  }
+
+#else /* USE_GNUTLS */
 		  case DKIM_HASHTYPE_SHA1:
 		  {
 			struct dkim_sha1 *sha1;
@@ -1252,7 +1424,7 @@ dkim_canon_runheaders(DKIM *dkim)
 			break;
 		  }
 
-#ifdef SHA256_DIGEST_LENGTH
+# ifdef HAVE_SHA256
 		  case DKIM_HASHTYPE_SHA256:
 		  {
 			struct dkim_sha256 *sha256;
@@ -1265,7 +1437,8 @@ dkim_canon_runheaders(DKIM *dkim)
 
 			break;
 		  }
-#endif /* SHA256_DIGEST_LENGTH */
+# endif /* HAVE_SHA256 */
+#endif /* USE_GNUTLS */
 
 		  default:
 			assert(0);
@@ -1341,6 +1514,41 @@ dkim_canon_signature(DKIM *dkim, struct dkim_header *hdr)
 		/* now close it */
 		switch (cur->canon_hashtype)
 		{
+#ifdef USE_GNUTLS
+		  case DKIM_HASHTYPE_SHA1:
+		  case DKIM_HASHTYPE_SHA256:
+		  {
+			int alg;
+			struct dkim_sha *sha;
+
+			sha = (struct dkim_sha *) cur->canon_hash;
+
+			if (cur->canon_hashtype == DKIM_HASHTYPE_SHA1)
+				alg = GNUTLS_DIG_SHA1;
+			else
+				alg = GNUTLS_DIG_SHA256;
+
+			sha->sha_outlen = gnutls_hash_get_len(alg);
+
+			sha->sha_out = DKIM_MALLOC(dkim, sha->sha_outlen);
+			if (sha->sha_out == NULL)
+			{
+				dkim_error(dkim, "unable to allocate %u bytes",
+				           sha->sha_outlen);
+				return DKIM_STAT_NORESOURCE;
+			}
+
+			gnutls_hash_output(sha->sha_hd, sha->sha_out);
+
+			if (sha->sha_tmpfd != -1)
+			{
+				close(sha->sha_tmpfd);
+				sha->sha_tmpfd = -1;
+			}
+
+			break;
+		  }
+#else /* USE_GNUTLS */
 		  case DKIM_HASHTYPE_SHA1:
 		  {
 			struct dkim_sha1 *sha1;
@@ -1354,7 +1562,7 @@ dkim_canon_signature(DKIM *dkim, struct dkim_header *hdr)
 			break;
 		  }
 
-#ifdef SHA256_DIGEST_LENGTH
+# ifdef HAVE_SHA256
 		  case DKIM_HASHTYPE_SHA256:
 		  {
 			struct dkim_sha256 *sha256;
@@ -1367,7 +1575,8 @@ dkim_canon_signature(DKIM *dkim, struct dkim_header *hdr)
 
 			break;
 		  }
-#endif /* SHA256_DIGEST_LENGTH */
+# endif /* HAVE_SHA256 */
+#endif /* USE_GNUTLS */
 
 		  default:
 			assert(0);
@@ -1526,7 +1735,7 @@ dkim_canon_bodychunk(DKIM *dkim, u_char *buf, size_t buflen)
 						else
 						{
 							dkim_canon_buffer(cur,
-							                  "\r",
+							                  (u_char *) "\r",
 							                  1);
 						}
 					}
@@ -1731,6 +1940,37 @@ dkim_canon_closebody(DKIM *dkim)
 		/* finalize */
 		switch (cur->canon_hashtype)
 		{
+#ifdef USE_GNUTLS
+		  case DKIM_HASHTYPE_SHA1:
+		  case DKIM_HASHTYPE_SHA256:
+		  {
+			int alg;
+			u_int diglen;
+			struct dkim_sha *sha;
+
+			sha = (struct dkim_sha *) cur->canon_hash;
+
+			if (cur->canon_hashtype == DKIM_HASHTYPE_SHA1)
+				alg = GNUTLS_DIG_SHA1;
+			else
+				alg = GNUTLS_DIG_SHA256;
+
+			diglen = gnutls_hash_get_len(alg);
+
+			sha->sha_out = DKIM_MALLOC(dkim, diglen);
+			if (sha->sha_out == NULL)
+			{
+				dkim_error(dkim, "unable to allocate %u bytes",
+				           diglen);
+				return DKIM_STAT_NORESOURCE;
+			}
+
+			gnutls_hash_output(sha->sha_hd, sha->sha_out);
+			sha->sha_outlen = diglen;
+
+			break;
+		  }
+#else /* USE_GNUTLS */
 		  case DKIM_HASHTYPE_SHA1:
 		  {
 			struct dkim_sha1 *sha1;
@@ -1744,7 +1984,7 @@ dkim_canon_closebody(DKIM *dkim)
 			break;
 		  }
 
-#ifdef SHA256_DIGEST_LENGTH
+# ifdef HAVE_SHA256
 		  case DKIM_HASHTYPE_SHA256:
 		  {
 			struct dkim_sha256 *sha256;
@@ -1757,7 +1997,8 @@ dkim_canon_closebody(DKIM *dkim)
 
 			break;
 		  }
-#endif /* SHA256_DIGEST_LENGTH */
+# endif /* HAVE_SHA256 */
+#endif /* USE_GNUTLS */
 
 		  default:
 			assert(0);
@@ -1794,6 +2035,19 @@ dkim_canon_getfinal(DKIM_CANON *canon, u_char **digest, size_t *dlen)
 
 	switch (canon->canon_hashtype)
 	{
+#ifdef USE_GNUTLS
+	  case DKIM_HASHTYPE_SHA1:
+	  case DKIM_HASHTYPE_SHA256:
+	  {
+		struct dkim_sha *sha;
+
+		sha = (struct dkim_sha *) canon->canon_hash;
+		*digest = sha->sha_out;
+		*dlen = sha->sha_outlen;
+
+		return DKIM_STAT_OK;
+	  }
+#else /* USE_GNUTLS */
 	  case DKIM_HASHTYPE_SHA1:
 	  {
 		struct dkim_sha1 *sha1;
@@ -1805,7 +2059,7 @@ dkim_canon_getfinal(DKIM_CANON *canon, u_char **digest, size_t *dlen)
 		return DKIM_STAT_OK;
 	  }
 
-#ifdef SHA256_DIGEST_LENGTH
+# ifdef HAVE_SHA256
 	  case DKIM_HASHTYPE_SHA256:
 	  {
 		struct dkim_sha256 *sha256;
@@ -1816,7 +2070,8 @@ dkim_canon_getfinal(DKIM_CANON *canon, u_char **digest, size_t *dlen)
 
 		return DKIM_STAT_OK;
 	  }
-#endif /* SHA256_DIGEST_LENGTH */
+# endif /* HAVE_SHA256 */
+#endif /* USE_GNUTLS */
 
 	  default:
 		assert(0);
