@@ -2,7 +2,7 @@
 **  Copyright (c) 2005-2009 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 **
-**  Copyright (c) 2009, 2010, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2009-2011, The OpenDKIM Project.  All rights reserved.
 */
 
 #ifndef _DKIM_H_
@@ -13,13 +13,17 @@ extern "C" {
 #endif /* __cplusplus */
 
 #ifndef lint
-static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.27 2010/05/20 18:39:10 cm-msk Exp $";
+static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.36.2.1 2010/10/27 21:43:08 cm-msk Exp $";
 #endif /* !lint */
 
 /* system includes */
 #include <sys/types.h>
 #include <sys/param.h>
-#include <stdbool.h>
+#include <sys/time.h>
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#endif /* HAVE_STDBOOL_H */
+#include <inttypes.h>
 
 /*
 **  version -- 0xrrMMmmpp
@@ -30,7 +34,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.27 2010/05/20 18:39:10 cm-msk Exp
 **  	pp == patch number
 */
 
-#define	OPENDKIM_LIB_VERSION	0x01020000
+#define	OPENDKIM_LIB_VERSION	0x02030200
 
 #ifdef __STDC__
 # ifndef __P
@@ -141,6 +145,19 @@ typedef int DKIM_SIGERROR;
 #define	DKIM_SIGERROR_KEYTYPEUNKNOWN	41	/* key type unknown */
 #define	DKIM_SIGERROR_KEYREVOKED	42	/* key revoked */
 #define	DKIM_SIGERROR_KEYDECODE		43	/* key couldn't be decoded */
+#define	DKIM_SIGERROR_MISSING_V		44	/* v= tag missing */
+#define	DKIM_SIGERROR_EMPTY_V		45	/* v= tag empty */
+
+/* generic DNS error codes */
+#define	DKIM_DNS_ERROR		(-1)		/* error in transit */
+#define	DKIM_DNS_SUCCESS	0		/* reply available */
+#define	DKIM_DNS_NOREPLY	1		/* reply not available (yet) */
+#define	DKIM_DNS_EXPIRED	2		/* no reply, query expired */
+
+#define	DKIM_DNSSEC_UNKNOWN	(-1)		/* not checked */
+#define	DKIM_DNSSEC_BOGUS	0		/* forged */
+#define	DKIM_DNSSEC_INSECURE	1		/* unprotected */
+#define	DKIM_DNSSEC_SECURE	2		/* verified */
 
 /*
 **  DKIM_CANON -- canonicalization method
@@ -219,7 +236,7 @@ typedef int dkim_policy_t;
 
 #define DKIM_PRESULT_NONE		(-1)	/* none/undefined */
 #define DKIM_PRESULT_NXDOMAIN		0	/* domain does not exist */
-#define DKIM_PRESULT_AUTHOR		1	/* ADSP found for author */
+#define DKIM_PRESULT_FOUND		1	/* ADSP query succeeded */
 
 /*
 **  DKIM_MODE -- mode of a handle
@@ -245,7 +262,7 @@ typedef int dkim_opts_t;
 #define	DKIM_OPTS_TIMEOUT	2
 #define	DKIM_OPTS_SENDERHDRS	3
 #define	DKIM_OPTS_SIGNHDRS	4
-#define	DKIM_OPTS_VERSION	5	/* unused */
+#define	DKIM_OPTS_OVERSIGNHDRS	5
 #define	DKIM_OPTS_QUERYMETHOD	6
 #define	DKIM_OPTS_QUERYINFO	7
 #define	DKIM_OPTS_FIXEDTIME	8
@@ -255,25 +272,34 @@ typedef int dkim_opts_t;
 #define	DKIM_OPTS_CLOCKDRIFT	12
 #define	DKIM_OPTS_MUSTBESIGNED	13
 
-#define	DKIM_LIBFLAGS_NONE		0x000
-#define	DKIM_LIBFLAGS_TMPFILES		0x001
-#define	DKIM_LIBFLAGS_KEEPFILES		0x002
-#define	DKIM_LIBFLAGS_SIGNLEN		0x004
-#define DKIM_LIBFLAGS_CACHE		0x008
-#define DKIM_LIBFLAGS_ZTAGS		0x010
-#define DKIM_LIBFLAGS_DELAYSIGPROC	0x020
-#define DKIM_LIBFLAGS_EOHCHECK		0x040
-#define DKIM_LIBFLAGS_ACCEPTV05		0x080
-#define DKIM_LIBFLAGS_FIXCRLF		0x100
-#define DKIM_LIBFLAGS_ACCEPTDK		0x200
-#define DKIM_LIBFLAGS_BADSIGHANDLES	0x400
-#define DKIM_LIBFLAGS_VERIFYONE		0x800
+#define	DKIM_LIBFLAGS_NONE		0x0000
+#define	DKIM_LIBFLAGS_TMPFILES		0x0001
+#define	DKIM_LIBFLAGS_KEEPFILES		0x0002
+#define	DKIM_LIBFLAGS_SIGNLEN		0x0004
+#define DKIM_LIBFLAGS_CACHE		0x0008
+#define DKIM_LIBFLAGS_ZTAGS		0x0010
+#define DKIM_LIBFLAGS_DELAYSIGPROC	0x0020
+#define DKIM_LIBFLAGS_EOHCHECK		0x0040
+#define DKIM_LIBFLAGS_ACCEPTV05		0x0080
+#define DKIM_LIBFLAGS_FIXCRLF		0x0100
+#define DKIM_LIBFLAGS_ACCEPTDK		0x0200
+#define DKIM_LIBFLAGS_BADSIGHANDLES	0x0400
+#define DKIM_LIBFLAGS_VERIFYONE		0x0800
+#define DKIM_LIBFLAGS_STRICTHDRS	0x1000
 
 #define	DKIM_LIBFLAGS_DEFAULT		DKIM_LIBFLAGS_NONE
 
 #define DKIM_REP_DEFREJECT	1001
 #define DKIM_REP_ROOT		"al.dkim-reputation.org"
 
+/*
+**  DKIM_PFLAG -- policy flags
+*/
+
+#define	DKIM_PFLAG_UNUSED1	0x01	/* no longer used */
+#define	DKIM_PFLAG_UNUSED2	0x02	/* no longer used */
+#define	DKIM_PFLAG_ATPS		0x04	/* atps */
+
 /*
 **  DKIM_DNSSEC -- results of DNSSEC queries
 */
@@ -283,6 +309,16 @@ typedef int dkim_opts_t;
 #define DKIM_DNSSEC_INSECURE	1
 #define DKIM_DNSSEC_SECURE	2
 
+/*
+**  DKIM_ATPS -- ATPS result codes
+*/
+
+#define	DKIM_ATPS_UNKNOWN	(-1)
+#define	DKIM_ATPS_NOTFOUND	0
+#define	DKIM_ATPS_FOUND		1
+
+typedef int dkim_atps_t;
+
 /*
 **  DKIM_LIB -- library handle
 */
@@ -341,19 +377,6 @@ struct dkim_hdrdiff
 **  PROTOTYPES
 */
 
-/*
-**  DKIM_SET_TRUST_ANCHOR -- set trust anchor file path
-**
-**  Parameters:
-**  	lib -- DKIM_LIB handle to update
-**  	tafile -- trust anchor file path
-** 
-**  Return value:
-**  	A DKIM_STAT constant.
-*/
-
-extern DKIM_STAT dkim_set_trust_anchor __P((DKIM_LIB *lib, char *tafile));
-
 /*
 **  DKIM_INIT -- initialize the DKIM package
 **
@@ -405,9 +428,10 @@ extern void dkim_close __P((DKIM_LIB *lib));
 **  	updated.
 */
 
-extern DKIM *dkim_sign __P((DKIM_LIB *libhandle, const char *id,
+extern DKIM *dkim_sign __P((DKIM_LIB *libhandle, const unsigned char *id,
                             void *memclosure, const dkim_sigkey_t secretkey,
-                            const char *selector, const char *domain,
+                            const unsigned char *selector,
+                            const unsigned char *domain,
                             dkim_canon_t hdr_canon_alg,
                             dkim_canon_t body_canon_alg,
                             dkim_alg_t sign_alg,
@@ -428,7 +452,7 @@ extern DKIM *dkim_sign __P((DKIM_LIB *libhandle, const char *id,
 **  	updated.
 */
 
-extern DKIM *dkim_verify __P((DKIM_LIB *libhandle, const char *id,
+extern DKIM *dkim_verify __P((DKIM_LIB *libhandle, const unsigned char *id,
                               void *memclosure, DKIM_STAT *statp));
 
 /*
@@ -568,6 +592,18 @@ extern DKIM_STAT dkim_policy_syntax __P((DKIM *dkim, u_char *str, size_t len));
 
 extern DKIM_STAT dkim_sig_syntax __P((DKIM *dkim, u_char *str, size_t len));
 
+/*
+**  DKIM_GETID -- retrieve "id" pointer from a handle
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**
+**  Return value:
+**  	The "id" pointer from inside the handle, stored when it was created.
+*/
+
+extern const char *dkim_getid __P((DKIM *dkim));
+
 /*
 **  DKIM_GETCACHESTATS -- retrieve cache statistics
 **
@@ -689,7 +725,7 @@ extern DKIM_STAT dkim_getsighdr_d __P((DKIM *dkim, size_t initial,
 **  	appeared in that list.
 */
 
-extern _Bool dkim_sig_hdrsigned __P((DKIM_SIGINFO *sig, char *hdr));
+extern _Bool dkim_sig_hdrsigned __P((DKIM_SIGINFO *sig, u_char *hdr));
 
 /*
 **  DKIM_SIG_GETDNSSEC -- retrieve DNSSEC results for a signature
@@ -701,7 +737,7 @@ extern _Bool dkim_sig_hdrsigned __P((DKIM_SIGINFO *sig, char *hdr));
 **  	A DKIM_DNSSEC_* constant.
 */
 
-extern u_int dkim_sig_getdnssec __P((DKIM_SIGINFO *sig));
+extern int dkim_sig_getdnssec __P((DKIM_SIGINFO *sig));
 
 /*
 **  DKIM_SIG_GETREPORTINFO -- retrieve reporting information from a key
@@ -747,7 +783,7 @@ extern DKIM_STAT dkim_sig_getreportinfo __P((DKIM *dkim, DKIM_SIGINFO *sig,
 */
 
 extern DKIM_STAT dkim_sig_getidentity __P((DKIM *dkim, DKIM_SIGINFO *sig,
-                                           char *val, size_t vallen));
+                                           u_char *val, size_t vallen));
 
 /*
 **  DKIM_SIG_GETCANONLEN -- report number of (canonicalized) body bytes that
@@ -854,7 +890,7 @@ extern DKIM_STAT dkim_sig_getsignalg __P((DKIM_SIGINFO *sig, dkim_alg_t *alg));
 **  	A DKIM_STAT value.
 */
 
-extern DKIM_STAT dkim_sig_getsigntime __P((DKIM_SIGINFO *sig, time_t *when));
+extern DKIM_STAT dkim_sig_getsigntime __P((DKIM_SIGINFO *sig, uint64_t *when));
 
 /*
 **  DKIM_SIG_GETSELECTOR -- retrieve selector used to generate the signature
@@ -929,7 +965,7 @@ extern const void *dkim_get_user_context __P((DKIM *dkim));
 **  	or 0 if no such header was found or the value in it was unusable
 */
 
-extern time_t dkim_get_msgdate __P((DKIM *dkim));
+extern uint64_t dkim_get_msgdate __P((DKIM *dkim));
 
 /*
 **  DKIM_GETMODE -- return the mode (signing, verifying, etc.) of a handle
@@ -956,6 +992,31 @@ extern int dkim_getmode __P((DKIM *dkim));
 
 extern u_char *dkim_getdomain __P((DKIM *dkim));
 
+/*
+**  DKIM_GETUSER -- retrieve sending user (local-part) from a DKIM context
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**
+**  Return value:
+**  	Pointer to the apparent sending user (local-part) or NULL if not known.
+*/
+
+extern u_char *dkim_getuser __P((DKIM *dkim));
+
+/*
+**  DKIM_GET_SIGNER -- get DKIM signature's signer
+**
+**  Parameters:
+**  	dkim -- DKIM signing handle
+**
+**  Parameters:
+**  	Pointer to a buffer containing the signer previously requested,
+**  	or NULL if none.
+*/
+
+extern unsigned char *dkim_get_signer __P((DKIM *dkim));
+
 /*
 **  DKIM_SET_SIGNER -- set DKIM signature's signer
 **
@@ -967,7 +1028,7 @@ extern u_char *dkim_getdomain __P((DKIM *dkim));
 **  	A DKIM_STAT_* constant.
 */
 
-extern DKIM_STAT dkim_set_signer __P((DKIM *dkim, const char *signer));
+extern DKIM_STAT dkim_set_signer __P((DKIM *dkim, const u_char *signer));
 
 /*
 **  DKIM_SET_DNS_CALLBACK -- set the DNS wait callback
@@ -1153,12 +1214,37 @@ extern const char *dkim_sig_geterrorstr __P((DKIM_SIGERROR sigerr));
 
 extern void dkim_sig_ignore __P((DKIM_SIGINFO *siginfo));
 
+/*
+**  DKIM_POLICY_STATE_NEW -- initialize and return a DKIM policy state handle
+**
+**  Parameters:
+**  	dkim -- DKIM handle from which to do an allocation
+**
+**  Return value:
+**  	A DKIM_PSTATE handle, or NULL on failure.
+*/
+
+extern DKIM_PSTATE *dkim_policy_state_new __P((DKIM *dkim));
+
+/*
+**  DKIM_POLICY_STATE_FREE -- destroy a DKIM policy state handle
+**
+**  Parameters:
+**  	pstate -- previously allocated policy state handle
+**
+**  Return value:
+**  	None.
+*/
+
+extern void dkim_policy_state_free __P((DKIM_PSTATE *pstate));
+
 /*
 **  DKIM_POLICY -- parse policy associated with the sender's domain
 **
 **  Parameters:
 **  	dkim -- DKIM handle
 **  	pcode -- discovered policy (returned)
+**  	pflags -- discovered policy flags (returned)
 **  	pstate -- state, for re-entrancy (updated; can be NULL)
 **
 **  Return value:
@@ -1166,7 +1252,7 @@ extern void dkim_sig_ignore __P((DKIM_SIGINFO *siginfo));
 */
 
 extern DKIM_STAT dkim_policy __P((DKIM *dkim, dkim_policy_t *pcode,
-                                  DKIM_PSTATE *pstate));
+                                  u_int *pflags, DKIM_PSTATE *pstate));
 
 /*
 **  DKIM_POLICY_GETDNSSEC -- retrieve DNSSEC results for a policy
@@ -1178,7 +1264,7 @@ extern DKIM_STAT dkim_policy __P((DKIM *dkim, dkim_policy_t *pcode,
 **  	A DKIM_DNSSEC_* constant.
 */
 
-extern u_int dkim_policy_getdnssec __P((DKIM *dkim));
+extern int dkim_policy_getdnssec __P((DKIM *dkim));
 
 /*
 **  DKIM_POLICY_GETREPORTINFO -- retrieve reporting information from policy
@@ -1306,7 +1392,7 @@ extern const char *dkim_getpolicystr __P((int policy));
 **  	A DKIM_STAT_* constant.
 */
 
-extern DKIM_STAT dkim_ohdrs __P((DKIM *dkim, DKIM_SIGINFO *sig, char **ptrs,
+extern DKIM_STAT dkim_ohdrs __P((DKIM *dkim, DKIM_SIGINFO *sig, u_char **ptrs,
                                  int *pcnt));
 
 /*
@@ -1314,6 +1400,7 @@ extern DKIM_STAT dkim_ohdrs __P((DKIM *dkim, DKIM_SIGINFO *sig, char **ptrs,
 **
 **  Parameters:
 **  	dkim -- DKIM handle
+**  	canon -- canonicalization mode in use
 **  	maxcost -- maximum "cost" of changes to be reported
 **  	ohdrs -- original headers, presumably extracted from a "z" tag
 **  	nohdrs -- number of headers at "ohdrs" available
@@ -1328,9 +1415,10 @@ extern DKIM_STAT dkim_ohdrs __P((DKIM *dkim, DKIM_SIGINFO *sig, char **ptrs,
 **  	destroyed.
 */
 
-extern DKIM_STAT dkim_diffheaders __P((DKIM *dkim, int maxcost, char **ohdrs,
-                                       int nohdrs, struct dkim_hdrdiff **out,
-                                       int *nout));
+extern DKIM_STAT dkim_diffheaders __P((DKIM *dkim, dkim_canon_t canon,
+                                       int maxcost,
+                                       char **ohdrs, int nohdrs,
+                                       struct dkim_hdrdiff **out, int *nout));
 
 /*
 **  DKIM_GETPARTIAL -- return a DKIM handle's "body length tag" flag
@@ -1406,7 +1494,7 @@ extern DKIM_STAT dkim_get_reputation __P((DKIM *dkim, DKIM_SIGINFO *sig,
 **  	0 on success; other on error (see source)
 */
 
-extern int dkim_mail_parse __P((char *addr, char **user, char **domain));
+extern int dkim_mail_parse __P((u_char *addr, u_char **user, u_char **domain));
 
 /*
 **  DKIM_SSL_VERSION -- return the version of the OpenSSL library against
@@ -1437,11 +1525,12 @@ extern unsigned long dkim_ssl_version __P((void));
 #define DKIM_FEATURE_PARSE_TIME		2
 #define DKIM_FEATURE_QUERY_CACHE	3
 #define DKIM_FEATURE_SHA256		4
-#define DKIM_FEATURE_ASYNC_DNS		5
+#define DKIM_FEATURE_OVERSIGN		5
 #define DKIM_FEATURE_DNSSEC		6
 #define DKIM_FEATURE_RESIGN		7
+#define DKIM_FEATURE_ATPS		8
 
-#define	DKIM_FEATURE_MAX		7
+#define	DKIM_FEATURE_MAX		8
 
 extern _Bool dkim_libfeature __P((DKIM_LIB *lib, u_int fc));
 
@@ -1505,6 +1594,7 @@ extern int dkim_test_adsp __P((DKIM_LIB *, const char *, dkim_policy_t *,
 **  	domain -- domain name
 **  	key -- private key to verify (PEM format)
 **  	keylen -- size of private key
+**  	dnssec -- DNSSEC result (may be NULL)
 **  	err -- error buffer (may be NULL)
 **  	errlen -- size of error buffer
 **
@@ -1515,7 +1605,7 @@ extern int dkim_test_adsp __P((DKIM_LIB *, const char *, dkim_policy_t *,
 */
 
 extern int dkim_test_key __P((DKIM_LIB *, char *, char *, char *, size_t,
-                              char *, size_t));
+                              int *, char *, size_t));
 
 /*
 **  DKIM_SIG_GETTAGVALUE -- retrieve a tag's value from a signature or its key
@@ -1537,7 +1627,25 @@ extern int dkim_test_key __P((DKIM_LIB *, char *, char *, char *, size_t,
 **  	necessarily been vetted in any way.  Caveat emptor.
 */
 
-extern u_char *dkim_sig_gettagvalue __P((DKIM_SIGINFO *, _Bool, char *));
+extern u_char *dkim_sig_gettagvalue __P((DKIM_SIGINFO *, _Bool, u_char *));
+
+/*
+**  DKIM_SIG_GETSIGNEDHDRS -- retrieve the signed header fields covered by
+**                            a signature that passed
+**
+**  Parameters:
+**  	dkim -- DKIM instance
+**  	sig -- signature
+**  	hdrs -- rectangular array of header field strings
+**  	hdrlen -- length of each element of "hdrs"
+**  	nhdrs -- size of "hdrs" array (updated)
+**
+**  Return value:
+**  	A DKIM_STAT_* constant.
+*/
+
+extern DKIM_STAT dkim_sig_getsignedhdrs __P((DKIM *, DKIM_SIGINFO *,
+                                             u_char *, size_t, u_int *));
 
 /*
 **  DKIM_STRLCPY -- size-bounded strcpy()
@@ -1586,6 +1694,132 @@ extern size_t dkim_strlcat __P((char *, const char *, ssize_t));
 
 extern int dkim_qp_decode __P((unsigned char *, unsigned char *, int));
 
+/*
+**  DKIM_DNS_SET_QUERY_SERVICE -- stores a handle representing the DNS
+**                                query service to be used, returning any
+**                                previous handle
+**
+**  Parameters:
+**  	lib -- DKIM library handle
+**  	h -- handle to be used
+**
+**  Return value:
+**  	Previously stored handle, or NULL if none.
+*/
+
+extern void *dkim_dns_set_query_service __P((DKIM_LIB *, void *));
+
+/*
+**  DKIM_DNS_SET_QUERY_START -- stores a pointer to a query start function
+**
+**  Parameters:
+**  	lib -- DKIM library handle
+**  	func -- function to use to start queries
+**
+**  Return value:
+**  	None.
+**
+**  Notes:
+**  	"func" should match the following prototype:
+**  		returns int (status)
+**  		void *dns -- receives handle stored by
+**  		             dkim_dns_set_query_service()
+**  		int type -- DNS RR query type (C_IN assumed)
+**  		char *query -- question to ask
+**  		char *buf -- buffer into which to write reply
+**  		size_t buflen -- size of buf
+**  		void **qh -- returned query handle
+*/
+
+extern void dkim_dns_set_query_start __P((DKIM_LIB *,
+                                          int (*)(void *, int,
+                                                  unsigned char *,
+                                                  unsigned char *,
+                                                  size_t, void **)));
+
+/*
+**  DKIM_DNS_SET_QUERY_CANCEL -- stores a pointer to a query cancel function
+**
+**  Parameters:
+**  	lib -- DKIM library handle
+**  	func -- function to use to cancel running queries
+**
+**  Return value:
+**  	None.
+**
+**  Notes:
+**  	"func" should match the following prototype:
+**  		returns int (status)
+**  		void *dns -- DNS service handle
+**  		void *qh -- query handle to be canceled
+*/
+
+extern void dkim_dns_set_query_cancel __P((DKIM_LIB *,
+                                           int (*)(void *, void *)));
+
+/*
+**  DKIM_DNS_SET_QUERY_WAITREPLY -- stores a pointer to wait for a DNS reply
+**
+**  Parameters:
+**  	lib -- DKIM library handle
+**  	func -- function to use to wait for a reply
+**
+**  Return value:
+**  	None.
+**
+**  Notes:
+**  	"func" should match the following prototype:
+**  		returns int (status)
+**  		void *dns -- DNS service handle
+**  		void *qh -- handle of query that has completed
+**  		struct timeval *timeout -- how long to wait
+**  		size_t *bytes -- bytes returned
+**  		int *error -- error code returned
+**  		int *dnssec -- DNSSEC status returned
+*/
+
+extern void dkim_dns_set_query_waitreply __P((DKIM_LIB *,
+                                              int (*)(void *, void *,
+                                                      struct timeval *,
+                                                      size_t *, int *,
+                                                      int *)));
+
+/*
+**  DKIM_ATPS_CHECK -- check for Authorized Third Party Signing
+**
+**  Parameters:
+**  	dkim -- DKIM message handle
+**  	sig -- signature information handle
+**  	timeout -- timeout (can be NULL)
+**  	res -- ATPS result code
+**
+**  Return value:
+**  	A DKIM_STAT_* constant.
+*/
+
+extern DKIM_STAT dkim_atps_check __P((DKIM *, DKIM_SIGINFO *,
+                                      struct timeval *, dkim_atps_t *res));
+
+/*
+**  DKIM_BASE32_ENCODE -- encode a string using base32
+**
+**  Parameters:
+**  	buf -- destination buffer
+**  	buflen -- bytes available at buf (updated)
+**  	data -- pointer to data to encode
+**  	size -- bytes at "data" to encode
+**
+**  Return value:
+**  	Length of encoding.
+**
+**  Notes:
+**  	buf should be at least a byte more than *buflen to hold the trailing
+**  	'\0'.
+**
+**  	*buflen is updated to count the number of bytes read from "data".
+*/
+
+extern int dkim_base32_encode __P((char *, size_t *, const void *, size_t));
 
 /* default list of sender headers */
 extern const u_char *dkim_default_senderhdrs[];
