@@ -4,13 +4,18 @@
 **
 **  Copyright (c) 2009, 2010, The OpenDKIM Project.  All rights reserved.
 **
-**  $Id: config.c,v 1.6 2010/02/23 23:32:23 cm-msk Exp $
+**  $Id: config.c,v 1.10.10.1 2010/10/27 21:43:09 cm-msk Exp $
 */
 
 #ifndef lint
-static char config_c_id[] = "@(#)$Id: config.c,v 1.6 2010/02/23 23:32:23 cm-msk Exp $";
+static char config_c_id[] = "@(#)$Id: config.c,v 1.10.10.1 2010/10/27 21:43:09 cm-msk Exp $";
 #endif /* !lint */
 
+/* for Solaris */
+#ifndef _REENTRANT
+# define _REENTRANT
+#endif /* _REENTRANT */
+
 /* system includes */
 #include <sys/types.h>
 #include <string.h>
@@ -106,13 +111,13 @@ config_load_level(char *file, struct configdef *def,
                   unsigned int *line, char *outpath, size_t outpathlen,
                   int level)
 {
-	int arg;
 	int n = -1;
 	int err = 0;
 	unsigned int myline = 0;
 	int value = -1;
 	FILE *in;
 	char *p;
+	char *s;
 	char *str = NULL;
 	struct config *new = NULL;
 	struct config *cur = NULL;
@@ -162,32 +167,46 @@ config_load_level(char *file, struct configdef *def,
 			}
 		}
 
-		arg = 0;
-
-		/* break down the arguments */
-		/* XXX -- need something better than strtok(), for quoting */
-		for (p = strtok(buf, " \t");
-		     err == 0 && p != NULL; 
-		     p = strtok(NULL, " \t"))
+		/* break down the line */
+		p = strtok_r(buf, " \t", &s);
+		if (p != NULL)
 		{
-			/* recognize the first? */
-			if (arg == 0)
+			/* recognize the directive? */
+			for (n = 0; ; n++)
 			{
-				for (n = 0; ; n++)
+				/* nope */
+				if (def[n].cd_name == NULL)
 				{
-					/* nope */
-					if (def[n].cd_name == NULL)
-					{
-						conf_error = CONF_UNRECOG;
-						err = 1;
-						break;
-					}
+					conf_error = CONF_UNRECOG;
+					err = 1;
+					break;
+			     	}
 
-					if (strcasecmp(def[n].cd_name, p) == 0)
-						break;
-				}
+				if (strcasecmp(def[n].cd_name, p) == 0)
+					break;
 			}
-			else if (str == NULL)
+
+			if (!err)
+			{
+				char *q;
+
+				/* skip leading whitespace on value */
+				for (p = s; *p == ' ' || *p == '\t'; p++)
+					continue;
+
+				/* ...and trim trailing whitespace */
+				q = p + strlen(p) - 1;
+				while (p <= q && (*q == '\t' || *q == ' '))
+					*q-- = '\0';
+			}
+
+			if (*p == '\0' && !err)
+			{
+				conf_error = CONF_MISSING;
+				err = 1;
+			}
+
+			if (!err)
 			{
 				char *q;
 
@@ -241,16 +260,14 @@ config_load_level(char *file, struct configdef *def,
 					return NULL;
 				}
 			}
-
-			arg++;
+		}
+		else
+		{
+			continue;			/* blank line */
 		}
 
-		/* no arguments */
-		if (arg == 0)
-			continue;
-
 		/* a parse error, or only one argument, is no good */
-		if (arg == 1 || err == 1)
+		if (err)
 		{
 			config_free(cur);
 
@@ -317,7 +334,7 @@ config_load_level(char *file, struct configdef *def,
 			break;
 
 		  case CONFIG_TYPE_BOOLEAN:
-			new->cfg_bool = (bool) value;
+			new->cfg_bool = (_Bool) value;
 			break;
 
 		  case CONFIG_TYPE_INTEGER:
@@ -558,7 +575,7 @@ config_get(struct config *head, const char *name, void *value, size_t size)
 			switch (cur->cfg_type)
 			{
 			  case CONFIG_TYPE_BOOLEAN:
-				if (size != sizeof(bool))
+				if (size != sizeof(_Bool))
 				{
 					conf_error = CONF_ILLEGAL;
 					return -1;
