@@ -2,20 +2,24 @@
 **  Copyright (c) 2006-2009 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  Copyright (c) 2009, 2010, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2009-2011, The OpenDKIM Project.  All rights reserved.
 **
-**  $Id: opendkim-config.h,v 1.17 2010/02/20 07:29:59 cm-msk Exp $
+**  $Id: opendkim-config.h,v 1.32.10.1 2010/10/27 21:43:09 cm-msk Exp $
 */
 
 #ifndef _DKIM_CONFIG_H_
 #define _DKIM_CONFIG_H_
 
 #ifndef lint
-static char dkim_config_h_id[] = "@(#)$Id: opendkim-config.h,v 1.17 2010/02/20 07:29:59 cm-msk Exp $";
+static char dkim_config_h_id[] = "@(#)$Id: opendkim-config.h,v 1.32.10.1 2010/10/27 21:43:09 cm-msk Exp $";
 #endif /* !lint */
 
 #include "build-config.h"
 
+/* system includes */
+#include <sys/types.h>
+
+/* macros */
 #ifndef FALSE
 # define FALSE	0
 #endif /* ! FALSE */
@@ -23,13 +27,18 @@ static char dkim_config_h_id[] = "@(#)$Id: opendkim-config.h,v 1.17 2010/02/20 0
 # define TRUE	1
 #endif /* ! TRUE */
 
+/* config definition */
 struct configdef dkimf_config[] =
 {
-	{ "ADSPDiscard",		CONFIG_TYPE_BOOLEAN,	FALSE },
+	{ "ADSPAction",			CONFIG_TYPE_STRING,	FALSE },
 	{ "ADSPNoSuchDomain",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "AllowSHA1Only",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "AlwaysAddARHeader",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "AlwaysSignHeaders",		CONFIG_TYPE_STRING,	FALSE },
+#ifdef _FFR_STATS
+	{ "AnonymousDomains",		CONFIG_TYPE_STRING,	FALSE },
+	{ "AnonymousStatistics",	CONFIG_TYPE_BOOLEAN,	FALSE },
+#endif /* _FFR_STATS */
 	{ "AuthservID",			CONFIG_TYPE_STRING,	FALSE },
 	{ "AuthservIDWithJobID",	CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "AutoRestart",		CONFIG_TYPE_BOOLEAN,	FALSE },
@@ -46,13 +55,17 @@ struct configdef dkimf_config[] =
 	{ "BogusPolicy",		CONFIG_TYPE_STRING,	FALSE },
 #endif /* USE_UNBOUND*/
 	{ "Canonicalization",		CONFIG_TYPE_STRING,	FALSE },
+	{ "CaptureUnknownErrors",	CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "ClockDrift",			CONFIG_TYPE_INTEGER,	FALSE },
+#ifdef _FFR_DEFAULT_SENDER
+	{ "DefaultSender",		CONFIG_TYPE_STRING,	FALSE },
+#endif /* _FFR_DEFAULT_SENDER */
 	{ "Diagnostics",		CONFIG_TYPE_BOOLEAN,	FALSE },
-#ifdef _FFR_ZTAGS
 	{ "DiagnosticDirectory",	CONFIG_TYPE_STRING,	FALSE },
-#endif /* _FFR_ZTAGS */
+	{ "DNSConnect",			CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "DNSTimeout",			CONFIG_TYPE_INTEGER,	FALSE },
 	{ "Domain",			CONFIG_TYPE_STRING,	FALSE },
+	{ "DomainKeysCompat",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "DontSignMailTo",		CONFIG_TYPE_STRING,	FALSE },
 	{ "EnableCoredumps",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "ExemptDomains",		CONFIG_TYPE_STRING,	FALSE },
@@ -71,6 +84,7 @@ struct configdef dkimf_config[] =
 	{ "InsecurePolicy",		CONFIG_TYPE_STRING,	FALSE },
 #endif /* USE_UNBOUND */
 	{ "InternalHosts",		CONFIG_TYPE_STRING,	FALSE },
+	{ "KeepAuthResults",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "KeepTemporaryFiles",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "KeyFile",			CONFIG_TYPE_STRING,	FALSE },
 	{ "KeyTable",			CONFIG_TYPE_STRING,	FALSE },
@@ -88,22 +102,33 @@ struct configdef dkimf_config[] =
 	{ "LocalADSP",			CONFIG_TYPE_STRING,	FALSE },
 	{ "LogWhy",			CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "MaximumHeaders",		CONFIG_TYPE_INTEGER,	FALSE },
-	{ "MaximumSignedBytes",		CONFIG_TYPE_INTEGER,	FALSE },
+#ifdef _FFR_MAXVERIFY
+	{ "MaximumSignaturesToVerify",	CONFIG_TYPE_INTEGER,	FALSE },
+#endif /* _FFR_MAXVERIFY */
 	{ "MacroList",			CONFIG_TYPE_STRING,	FALSE },
 	{ "MilterDebug",		CONFIG_TYPE_INTEGER,	FALSE },
 	{ "Minimum",			CONFIG_TYPE_STRING,	FALSE },
 	{ "MultipleSignatures",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "Mode",			CONFIG_TYPE_STRING,	FALSE },
 	{ "MTA",			CONFIG_TYPE_STRING,	FALSE },
+	{ "MTACommand",			CONFIG_TYPE_STRING,	FALSE },
 	{ "MustBeSigned",		CONFIG_TYPE_STRING,	FALSE },
+#ifdef _FFR_ADSP_LISTS
+	{ "NoDiscardableMailTo",	CONFIG_TYPE_STRING,	FALSE },
+#endif /* _FFR_ADSP_LISTS */
+	{ "NoHeaderB",			CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "OmitHeaders",		CONFIG_TYPE_STRING,	FALSE },
 	{ "On-BadSignature",		CONFIG_TYPE_STRING,	FALSE },
 	{ "On-Default",			CONFIG_TYPE_STRING,	FALSE },
 	{ "On-DNSError",		CONFIG_TYPE_STRING,	FALSE },
 	{ "On-InternalError",		CONFIG_TYPE_STRING,	FALSE },
 	{ "On-KeyNotFound",		CONFIG_TYPE_STRING,	FALSE },
+	{ "On-PolicyError",		CONFIG_TYPE_STRING,	FALSE },
 	{ "On-NoSignature",		CONFIG_TYPE_STRING,	FALSE },
 	{ "On-Security",		CONFIG_TYPE_STRING,	FALSE },
+#ifdef _FFR_OVERSIGN
+	{ "OverSignHeaders",		CONFIG_TYPE_STRING,	FALSE },
+#endif /* _FFR_OVERSIGN */
 	{ "PeerList",			CONFIG_TYPE_STRING,	FALSE },
 	{ "PidFile",			CONFIG_TYPE_STRING,	FALSE },
 #ifdef POPAUTH
@@ -126,6 +151,7 @@ struct configdef dkimf_config[] =
 	{ "ReportIntervalDB",		CONFIG_TYPE_STRING,	FALSE },
 #endif /* _FFR_REPORT_INTERVALS */
 	{ "ReportAddress",		CONFIG_TYPE_STRING,	FALSE },
+	{ "ReportBccAddress",		CONFIG_TYPE_STRING,	FALSE },
 #ifdef _FFR_DKIM_REPUTATION
 	{ "ReputationFail",		CONFIG_TYPE_INTEGER,	FALSE },
 	{ "ReputationPass",		CONFIG_TYPE_INTEGER,	FALSE },
@@ -133,13 +159,15 @@ struct configdef dkimf_config[] =
 	{ "ReputationRoot",		CONFIG_TYPE_STRING,	FALSE },
 #endif /* _FFR_DKIM_REPUTATION */
 	{ "RequiredHeaders",		CONFIG_TYPE_BOOLEAN,	FALSE },
+	{ "RequireSafeKeys",		CONFIG_TYPE_BOOLEAN,	FALSE },
 #ifdef _FFR_RESIGN
 	{ "ResignAll",			CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "ResignMailTo",		CONFIG_TYPE_STRING,	FALSE },
+#endif /* _FFR_RESIGN */
+	{ "ResolverTracing",		CONFIG_TYPE_BOOLEAN,	FALSE },
 #ifdef USE_LUA
 	{ "ScreenPolicyScript",		CONFIG_TYPE_STRING,	FALSE },
 #endif /* USE_LUA */
-#endif /* _FFR_RESIGN */
 	{ "Selector",			CONFIG_TYPE_STRING,	FALSE },
 #ifdef USE_LUA
 	{ "SetupPolicyScript",		CONFIG_TYPE_STRING,	FALSE },
@@ -158,10 +186,21 @@ struct configdef dkimf_config[] =
 	{ "SignatureTTL",		CONFIG_TYPE_INTEGER,	FALSE },
 	{ "SignHeaders",		CONFIG_TYPE_STRING,	FALSE },
 	{ "SigningTable",		CONFIG_TYPE_STRING,	FALSE },
+#ifdef VERIFY_DOMAINKEYS
+	{ "SingleAuthResult",		CONFIG_TYPE_BOOLEAN,	FALSE },
+#endif /* VERIFY_DOMAINKEYS */
 	{ "Socket",			CONFIG_TYPE_STRING,	FALSE },
 #ifdef _FFR_STATS
 	{ "Statistics",			CONFIG_TYPE_STRING,	FALSE },
+	{ "StatisticsName",		CONFIG_TYPE_STRING,	FALSE },
+# ifdef USE_LUA
+#  ifdef _FFR_STATSEXT
+	{ "StatisticsPolicyScript",	CONFIG_TYPE_STRING,	FALSE },
+#  endif /* _FFR_STATSEXT */
+# endif /* USE_LUA */
+	{ "StatisticsPrefix",		CONFIG_TYPE_STRING,	FALSE },
 #endif /* _FFR_STATS */
+	{ "StrictHeaders",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "StrictTestMode",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "SubDomains",			CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "Syslog",			CONFIG_TYPE_BOOLEAN,	FALSE },
@@ -174,14 +213,19 @@ struct configdef dkimf_config[] =
 #endif /* USE_UNBOUND */
 	{ "TrustSignaturesFrom",	CONFIG_TYPE_STRING,	FALSE },
 	{ "UMask",			CONFIG_TYPE_INTEGER,	FALSE },
+#ifdef USE_UNBOUND
+	{ "UnboundConfigFile",		CONFIG_TYPE_STRING,	FALSE },
+#endif /* USE_UNBOUND */
 	{ "UserID",			CONFIG_TYPE_STRING,	FALSE },
 #ifdef _FFR_VBR
 	{ "VBR-Certifiers",		CONFIG_TYPE_STRING,	FALSE },
+	{ "VBR-PurgeFields",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "VBR-TrustedCertifiers",	CONFIG_TYPE_STRING,	FALSE },
+	{ "VBR-TrustedCertifiersOnly",	CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "VBR-Type",			CONFIG_TYPE_STRING,	FALSE },
 #endif /* _FFR_VBR */
 	{ "X-Header",			CONFIG_TYPE_BOOLEAN,	FALSE },
-	{ NULL,				-1,			FALSE }
+	{ NULL,				(u_int) -1,		FALSE }
 };
 
 #endif /* _DKIM_CONFIG_H_ */
