@@ -2,22 +2,30 @@
 **  Copyright (c) 2008 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  Copyright (c) 2009, 2010, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2009-2011, The OpenDKIM Project.  All rights reserved.
 **
-**  $Id: opendkim-db.c,v 1.77.2.14 2010/07/14 19:38:39 cm-msk Exp $
+**  $Id: opendkim-db.c,v 1.101.10.1 2010/10/27 21:43:09 cm-msk Exp $
 */
 
 #ifndef lint
-static char opendkim_db_c_id[] = "@(#)$Id: opendkim-db.c,v 1.77.2.14 2010/07/14 19:38:39 cm-msk Exp $";
+static char opendkim_db_c_id[] = "@(#)$Id: opendkim-db.c,v 1.101.10.1 2010/10/27 21:43:09 cm-msk Exp $";
 #endif /* !lint */
 
 #include "build-config.h"
 
+/* for Solaris */
+#ifndef _REENTRANT
+# define _REENTRANT
+#endif /* ! _REENTRANT */
+
 /* system includes */
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/file.h>
-#include <stdbool.h>
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#endif /* HAVE_STDBOOL_H */
+#include <syslog.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
@@ -66,6 +74,10 @@ static char opendkim_db_c_id[] = "@(#)$Id: opendkim-db.c,v 1.77.2.14 2010/07/14
 /* macros */
 #define	BUFRSZ			1024
 #define	DEFARRAYSZ		16
+#ifdef _FFR_DB_HANDLE_POOLS
+# define DEFPOOLMAX		10
+#endif /* _FFR_DB_HANDLE_POOLS */
+#define DKIMF_DB_DEFASIZE	8
 #define DKIMF_DB_MODE		0644
 #define DKIMF_LDAP_MAXURIS	8
 #define DKIMF_LDAP_TIMEOUT	5
@@ -74,6 +86,7 @@ static char opendkim_db_c_id[] = "@(#)$Id: opendkim-db.c,v 1.77.2.14 2010/07/14
 #endif /* _FFR_LDAP_CACHING */
 
 #define	DKIMF_DB_IFLAG_FREEARRAY 0x01
+#define	DKIMF_DB_IFLAG_RECONNECT 0x02
 
 #ifndef FALSE
 # define FALSE			0
@@ -86,6 +99,8 @@ static char opendkim_db_c_id[] = "@(#)$Id: opendkim-db.c,v 1.77.2.14 2010/07/14
 # define MAX(x,y)	((x) > (y) ? (x) : (y))
 #endif /* ! MAX */
 
+#define STRORNULL(x)	((x)[0] == '\0' ? NULL : (x))
+
 #ifdef USE_DB
 # ifndef DB_NOTFOUND
 #  define DB_NOTFOUND		1
@@ -208,6 +223,7 @@ struct dkimf_db_ldap_cache
 struct dkimf_db_lua
 {
 	char *			lua_script;
+	size_t			lua_scriptlen;
 	char *			lua_error;
 };
 #endif /* USE_LUA */
@@ -238,6 +254,297 @@ struct dkimf_db_table dbtypes[] =
 
 static char *dkimf_db_ldap_param[DKIMF_LDAP_PARAM_MAX + 1];
 
+#ifdef _FFR_DB_HANDLE_POOLS
+struct handle_pool
+{
+	u_int		hp_dbtype;
+	u_int		hp_max;
+	u_int		hp_alloc;
+	u_int		hp_asize;
+	u_int		hp_count;
+	void *		hp_hdata;
+	void **		hp_handles;
+	pthread_mutex_t	hp_lock;
+	pthread_cond_t	hp_signal;
+};
+#endif /* _FFR_DB_HANDLE_POOLS */
+
+#ifdef _FFR_DB_HANDLE_POOLS
+/*
+**  DKIMF_DB_HP_NEW -- create a handle pool
+**
+**  Parameters:
+**  	type -- DB type
+**  	max -- maximum pool size
+**  	hdata -- data needed to make new handles
+**
+**  Return value:
+**  	Pointer to a newly-allocated handle pool, or NULL on error.
+*/
+
+static struct handle_pool *
+dkimf_db_hp_new(u_int type, u_int max, void *hdata)
+{
+	struct handle_pool *new;
+
+	new = (struct handle_pool *) malloc(sizeof *new);
+	if (new != NULL)
+	{
+		new->hp_alloc = 0;
+		new->hp_asize = 0;
+		new->hp_count = 0;
+		new->hp_dbtype = type;
+		new->hp_handles = NULL;
+		new->hp_hdata = hdata;
+		new->hp_max = max;
+		pthread_mutex_init(&new->hp_lock, NULL);
+		pthread_cond_init(&new->hp_signal, NULL);
+	}
+
+	return new;
+}
+
+/*
+**  DKIMF_DB_HP_FREE -- free a handle pool
+**
+**  Parameters:
+**  	pool -- bool to free up
+**
+**  Return value:
+**  	None.
+*/
+
+static void
+dkimf_db_hp_free(struct handle_pool *pool)
+{
+	u_int c;
+
+	assert(pool != NULL);
+
+	for (c = 0; c < pool->hp_count; c++)
+	{
+		switch (pool->hp_dbtype)
+		{
+#ifdef USE_ODBX
+		  case DKIMF_DB_TYPE_DSN:
+		  {
+			odbx_t *odbx;
+
+			odbx = (odbx_t *) pool->hp_handles[c];
+
+			(void) odbx_unbind(odbx);
+			(void) odbx_finish(odbx);
+			free(odbx);
+
+			break;
+		  }
+#endif /* USE_ODBX */
+
+		  default:
+			break;
+		}
+	}
+
+	pthread_mutex_destroy(&pool->hp_lock);
+	pthread_cond_destroy(&pool->hp_signal);
+	free(pool->hp_handles);
+	free(pool);
+}
+
+/*
+**  DKIMF_DB_HP_GET -- get a handle from a handle pool
+**
+**  Parameters:
+**  	pool -- pool from which to get a handle
+**  	err -- error code (returned)
+**
+**  Return value:
+**  	A handle appropriate to the associated DB type that is not currently
+**  	in use by another thread, or NULL on error.
+*/
+
+static void *
+dkimf_db_hp_get(struct handle_pool *pool, int *err)
+{
+	void *ret;
+
+	assert(pool != NULL);
+
+	pthread_mutex_lock(&pool->hp_lock);
+
+	for (;;)
+	{
+		/* if one is available, return it */
+		if (pool->hp_count > 0)
+		{
+			ret = pool->hp_handles[0];
+
+			if (pool->hp_count > 1)
+			{
+				memmove(&pool->hp_handles[0],
+				        &pool->hp_handles[1],
+				        sizeof(void *) * (pool->hp_count - 1));
+			}
+
+			pool->hp_count--;
+
+			pthread_mutex_unlock(&pool->hp_lock);
+
+			return ret;
+		}
+
+		/* if we can allocate one, do so */
+		if (pool->hp_alloc <= pool->hp_max)
+		{
+			switch (pool->hp_dbtype)
+			{
+#ifdef USE_ODBX
+			  case DKIMF_DB_TYPE_DSN:
+			  {
+				int dberr;
+				odbx_t *odbx;
+				struct dkimf_db_dsn *dsn;
+
+				dsn = (struct dkimf_db_dsn *) pool->hp_hdata;
+
+				dberr = odbx_init(&odbx,
+				                  STRORNULL(dsn->dsn_backend),
+				                  STRORNULL(dsn->dsn_host),
+				                  STRORNULL(dsn->dsn_port));
+
+				if (dberr < 0)
+				{
+					if (err != NULL)
+						*err = dberr;
+
+					(void) odbx_finish(odbx);
+					pthread_mutex_unlock(&pool->hp_lock);
+
+					return NULL;
+				}
+
+				dberr = odbx_bind(odbx,
+				                        STRORNULL(dsn->dsn_dbase),
+				                        STRORNULL(dsn->dsn_user),
+				                        STRORNULL(dsn->dsn_password),
+				                        ODBX_BIND_SIMPLE);
+				if (dberr < 0)
+				{
+					if (err != NULL)
+						*err = dberr;
+
+					(void) odbx_finish(odbx);
+					pthread_mutex_unlock(&pool->hp_lock);
+
+					return NULL;
+				}
+
+				ret = odbx;
+
+				break;
+			  }
+#endif /* USE_ODBX */
+
+			  default:
+				assert(0);
+				break;
+			}
+
+			pool->hp_alloc++;
+
+			pthread_mutex_unlock(&pool->hp_lock);
+
+			return ret;
+		}
+
+		/* already full; wait for one */
+		pthread_cond_wait(&pool->hp_signal, &pool->hp_lock);
+	}
+}
+
+/*
+**  DKIMF_DB_HP_DEAD -- report that a handle found in the pool was dead
+**
+**  Parameters:
+**  	pool -- handle pool to be updated
+**
+**  Return value:
+**  	None.
+**
+**  Notes:
+**  	The caller is expected to identify a dead handle and deallocate it.
+*/
+
+static void
+dkimf_db_hp_dead(struct handle_pool *pool)
+{
+	assert(pool != NULL);
+
+	pthread_mutex_lock(&pool->hp_lock);
+	pool->hp_alloc--;
+	pthread_cond_signal(&pool->hp_signal);
+	pthread_mutex_unlock(&pool->hp_lock);
+}
+
+/*
+**  DKIMF_DB_HP_PUT -- put a handle back into a handle pool after use
+**
+**  Parameters:
+**  	pool -- pool from which to get a handle
+**  	handle -- handle being returned
+**
+**  Return value:
+**  	None.
+*/
+
+static void
+dkimf_db_hp_put(struct handle_pool *pool, void *handle)
+{
+	assert(pool != NULL);
+	assert(handle != NULL);
+
+	pthread_mutex_lock(&pool->hp_lock);
+
+	/* need to grow the array? */
+	if (pool->hp_asize == pool->hp_count)
+	{
+		u_int newasz;
+
+		if (pool->hp_asize == 0)
+		{
+			newasz = DKIMF_DB_DEFASIZE;
+			pool->hp_handles = (void **) malloc(newasz * sizeof(void *));
+			assert(pool->hp_handles != NULL);
+		}
+		else
+		{
+			void **newa;
+
+			newasz = pool->hp_asize * 2;
+			newa = (void **) realloc(pool->hp_handles,
+			                         newasz * sizeof(void *));
+			assert(newa != NULL);
+			pool->hp_handles = newa;
+		}
+
+		pool->hp_asize = newasz;
+	}
+
+	/* append it */
+	pool->hp_handles[pool->hp_count] = handle;
+
+	/* increment the count */
+	pool->hp_count++;
+
+	/* signal any waiters */
+	pthread_cond_signal(&pool->hp_signal);
+
+	/* all done */
+	pthread_mutex_unlock(&pool->hp_lock);
+}
+
+#endif /* _FFR_DB_HANDLE_POOLS */
+
 #if (USE_SASL && USE_LDAP)
 /*
 **  DKIMF_DB_SASLINTERACT -- SASL binding interaction callback
@@ -379,14 +686,15 @@ dkimf_db_datasplit(char *buf, size_t buflen,
 	}
 
 	/* mark the ones that got no data */
-	if (ridx < reqnum - 1)
+	if (ridx < reqnum)
 	{
 		int c;
 
-		for (c = ridx + 1; c < reqnum; c++)
+		for (c = ridx; c < reqnum; c++)
 		{
-			ret = -1;
-			req[c].dbdata_buflen = 0;
+			if ((req[c].dbdata_flags & DKIMF_DB_DATA_OPTIONAL) == 0)
+				ret = -1;
+			req[c].dbdata_buflen = (size_t) -1;
 		}
 	}
 
@@ -500,7 +808,12 @@ dkimf_db_nextpunct(char *str)
 
 	for (p = str; *p != '\0'; p++)
 	{
-		if (isascii(*p) && ispunct(*p) && *p != '.')
+		if (*p == ':' ||
+		    *p == '/' ||
+		    *p == '@' ||
+		    *p == '+' ||
+		    *p == '=' ||
+		    *p == '?')
 			return p;
 	}
 
@@ -640,7 +953,6 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 
 	memset(new, '\0', sizeof(struct dkimf_db));
 
-	new->db_lock = lock;
 	new->db_flags = flags;
 	new->db_type = DKIMF_DB_TYPE_UNKNOWN;
 
@@ -695,6 +1007,30 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 		p++;
 	}
 
+	/* force DB accesses to be mutex-protected */
+	if (new->db_type == DKIMF_DB_TYPE_DSN)
+		new->db_flags |= DKIMF_DB_FLAG_MAKELOCK;
+
+	/* use provided lock, or create a new one if needed */
+	if (lock != NULL)
+	{
+		new->db_lock = lock;
+		new->db_flags &= ~DKIMF_DB_FLAG_MAKELOCK;
+	}
+	else if ((new->db_flags & DKIMF_DB_FLAG_MAKELOCK) != 0)
+	{
+		new->db_lock = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
+		if (new->db_lock == NULL)
+		{
+			if (err != NULL)
+				*err = strerror(errno);
+			free(new);
+			return -1;
+		}
+
+		pthread_mutex_init(new->db_lock, NULL);
+	}
+
 	switch (new->db_type)
 	{
 	  case DKIMF_DB_TYPE_CSL:
@@ -765,8 +1101,10 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 							dkimf_db_list_free(list);
 						free(new);
 						free(tmp);
+						free(new);
 						return -1;
 					}
+					dkimf_trimspaces(newl->db_list_key);
 
 					newl->db_list_value = strdup(q);
 					if (newl->db_list_value == NULL)
@@ -781,6 +1119,7 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 						free(new);
 						return -1;
 					}
+					dkimf_trimspaces(newl->db_list_value);
 
 					newl->db_list_next = NULL;
 
@@ -819,6 +1158,7 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 					free(new);
 					return -1;
 				}
+				dkimf_trimspaces(newl->db_list_key);
 
 				if (eq != NULL)
 				{
@@ -835,6 +1175,7 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 						free(new);
 						return -1;
 					}
+					dkimf_trimspaces(newl->db_list_value);
 				}
 				else
 				{
@@ -975,6 +1316,7 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 
 						return -1;
 					}
+					dkimf_trimspaces(newl->db_list_key);
 
 					newl->db_list_value = strdup(q);
 					if (newl->db_list_value == NULL)
@@ -987,6 +1329,7 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 							dkimf_db_list_free(list);
 						return -1;
 					}
+					dkimf_trimspaces(newl->db_list_value);
 
 					newl->db_list_next = NULL;
 
@@ -1013,6 +1356,7 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 					free(new);
 					return -1;
 				}
+				dkimf_trimspaces(newl->db_list_key);
 
 				if (value != NULL)
 				{
@@ -1029,6 +1373,7 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 						free(new);
 						return -1;
 					}
+					dkimf_trimspaces(newl->db_list_value);
 				}
 				else
 				{
@@ -1091,10 +1436,11 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 			reflags |= REG_ICASE;
 
 		memset(line, '\0', sizeof line);
-		end = NULL;
-		data = NULL;
 		while (fgets(line, BUFRSZ, f) != NULL)
 		{
+			end = NULL;
+			data = NULL;
+
 			for (p = line; *p != '\0'; p++)
 			{
 				if (*p == '\n' || *p == '#')
@@ -1131,12 +1477,11 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 					dkimf_db_relist_free(head);
 				fclose(f);
 				free(new);
+				free(newl);
 				return -1;
 			}
 
 			memset(patbuf, '\0', sizeof patbuf);
-			end = patbuf + sizeof patbuf;
-			patbuf[0] = '^';
 
 			if (!dkimf_mkregexp(line, patbuf, sizeof patbuf))
 			{
@@ -1177,6 +1522,7 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 					free(newl);
 					return -1;
 				}
+				dkimf_trimspaces(newl->db_relist_data);
 			}
 			else
 			{
@@ -1456,11 +1802,23 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 				*err = strerror(EINVAL);
 			free(dsn);
 			free(tmp);
+			free(new);
 			return -1;
 		}
 
-# define STRORNULL(x)	((x)[0] == '\0' ? NULL : (x))
-
+# ifdef _FFR_DB_HANDLE_POOLS
+		new->db_handle = dkimf_db_hp_new(new->db_type,
+		                                 DEFPOOLMAX, dsn);
+		if (new == NULL)
+		{
+			if (err != NULL)
+				*err = strerror(errno);
+			free(dsn);
+			free(tmp);
+			free(new);
+			return -1;
+		}
+# else /* _FFR_DB_HANDLE_POOLS */
 		/* create odbx handle */
 		dberr = odbx_init(&odbx,
 		                  STRORNULL(dsn->dsn_backend),
@@ -1483,7 +1841,7 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 		if (dberr < 0)
 		{
 			if (err != NULL)
-				*err = (char *) odbx_error(NULL, dberr);
+				*err = (char *) odbx_error(odbx, dberr);
 			(void) odbx_finish(odbx);
 			free(dsn);
 			free(tmp);
@@ -1493,10 +1851,13 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 
 		/* store handle */
 		new->db_handle = (void *) odbx;
+# endif /* _FFR_DB_HANDLE_POOLS */
+
 		new->db_data = (void *) dsn;
 
 		/* clean up */
 		free(tmp);
+
 		break;
 	  }
 #endif /* USE_ODBX */
@@ -1803,7 +2164,9 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 	  {
 		int fd;
 		ssize_t rlen;
+		char *tmp;
 		struct stat s;
+		struct dkimf_lua_script_result lres;
 		struct dkimf_db_lua *lua;
 
 		fd = open(p, O_RDONLY);
@@ -1832,8 +2195,8 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 		memset(lua, '\0', sizeof *lua);
 		new->db_data = (void *) lua;
 
-		lua->lua_script = (void *) malloc(s.st_size + 1);
-		if (lua->lua_script == NULL)
+		tmp = (void *) malloc(s.st_size + 1);
+		if (tmp == NULL)
 		{
 			if (err != NULL)
 				*err = strerror(errno);
@@ -1841,9 +2204,9 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 			close(fd);
 			return -1;
 		}
-		memset(lua->lua_script, '\0', s.st_size + 1);
+		memset(tmp, '\0', s.st_size + 1);
 
-		rlen = read(fd, lua->lua_script, s.st_size);
+		rlen = read(fd, tmp, s.st_size);
 		if (rlen < s.st_size)
 		{
 			if (err != NULL)
@@ -1853,11 +2216,27 @@ dkimf_db_open(DKIMF_DB *db, char *name, u_int flags, pthread_mutex_t *lock,
 				else
 					*err = "Read truncated";
 			}
-			free(lua->lua_script);
+			free(tmp);
 			free(new->db_data);
 			close(fd);
 			return -1;
 		}
+
+		close(fd);
+
+		/* try to compile it */
+		if (dkimf_lua_db_hook(tmp, 0, NULL, &lres, 
+		                      (void *) &lua->lua_script,
+		                      &lua->lua_scriptlen) != 0)
+		{
+			if (err != NULL)
+				*err = "Lua compilation error";
+			free(tmp);
+			free(new->db_data);
+			return -1;
+		}
+
+		free(tmp);
 	  }
 #endif /* USE_LUA */
 	}
@@ -2256,16 +2635,12 @@ dkimf_db_get(DKIMF_DB db, void *buf, size_t buflen,
 					matched = TRUE;
 			}
 
-			if (matched)
-			{
-				if ((db->db_flags & DKIMF_DB_FLAG_MATCHBOTH) == 0 ||
-				    list->db_list_value == NULL)
-					break;
-			}
+			if (!matched)
+				continue;
 
 			if ((db->db_flags & DKIMF_DB_FLAG_MATCHBOTH) == 0 ||
-			    reqnum == 0)
-				continue;
+			    reqnum == 0 || list->db_list_value == NULL)
+				break;
 
 			matched = FALSE;
 			assert(list->db_list_value != NULL);
@@ -2517,26 +2892,81 @@ dkimf_db_get(DKIMF_DB db, void *buf, size_t buflen,
 #ifdef USE_ODBX
 	  case DKIMF_DB_TYPE_DSN:
 	  {
+		_Bool reconnected = FALSE;
 		int err;
 		int fields;
 		int rescnt = 0;
 		int rowcnt = 0;
 		u_long elen;
 		odbx_result_t *result;
+		odbx_t *odbx = NULL;
 		struct dkimf_db_dsn *dsn;
 		char query[BUFRSZ];
 		char escaped[BUFRSZ];
 
 		dsn = (struct dkimf_db_dsn *) db->db_data;
 
+# ifdef _FFR_DB_HANDLE_POOLS
+		odbx = dkimf_db_hp_get((struct handle_pool *) db->db_handle,
+		                       &err);
+		if (odbx == NULL)
+		{
+			db->db_status = err;
+			return -1;
+		}
+# else /* _FFR_DB_HANDLE_POOLS */
+		if (db->db_lock != NULL)
+			(void) pthread_mutex_lock(db->db_lock);
+
+		/* see if we need to reopen */
+		if ((db->db_iflags & DKIMF_DB_IFLAG_RECONNECT) != 0)
+		{
+			err = odbx_init((odbx_t **) &db->db_handle,
+			                STRORNULL(dsn->dsn_backend),
+			                STRORNULL(dsn->dsn_host),
+			                STRORNULL(dsn->dsn_port));
+			if (err < 0)
+			{
+				db->db_status = err;
+				return -1;
+			}
+
+			err = odbx_bind((odbx_t *) db->db_handle,
+			                STRORNULL(dsn->dsn_dbase),
+		                        STRORNULL(dsn->dsn_user),
+		                        STRORNULL(dsn->dsn_password),
+		                        ODBX_BIND_SIMPLE);
+			if (err < 0)
+			{
+				(void) odbx_finish((odbx_t *) db->db_handle);
+				db->db_status = err;
+				return -1;
+			}
+
+			reconnected = TRUE;
+			db->db_iflags &= ~DKIMF_DB_IFLAG_RECONNECT;
+		}
+
+		odbx = (odbx_t *) db->db_handle;
+
+# endif /* _FFR_DB_HANDLE_POOLS */
+
 		memset(&elen, '\0', sizeof elen);
 		elen = sizeof escaped - 1;
-		err = odbx_escape((odbx_t *) db->db_handle, buf,
+		err = odbx_escape(odbx, buf,
 		                  (buflen == 0 ? strlen(buf) : buflen),
 		                  escaped, &elen);
 		if (err < 0)
 		{
 			db->db_status = err;
+			if (db->db_lock != NULL)
+				(void) pthread_mutex_unlock(db->db_lock);
+
+# ifdef _FFR_DB_HANDLE_POOLS
+			dkimf_db_hp_put((struct handle_pool *) db->db_handle,
+			                (void *) odbx);
+# endif /* _FFR_DB_HANDLE_POOLS */
+
 			return err;
 		}
 
@@ -2546,20 +2976,124 @@ dkimf_db_get(DKIMF_DB db, void *buf, size_t buflen,
 		         dsn->dsn_table,
 		         dsn->dsn_keycol, escaped);
 
-		err = odbx_query((odbx_t *) db->db_handle, query, 0);
+		err = odbx_query(odbx, query, 0);
 		if (err < 0)
 		{
+			int status;
+
 			db->db_status = err;
-			return err;
+
+			if (reconnected)
+			{
+				if (db->db_lock != NULL)
+					(void) pthread_mutex_unlock(db->db_lock);
+# ifdef _FFR_DB_HANDLE_POOLS
+				dkimf_db_hp_put((struct handle_pool *) db->db_handle,
+				                (void *) odbx);
+# endif /* _FFR_DB_HANDLE_POOLS */
+
+				return err;
+			}
+
+			status = odbx_error_type(odbx, err);
+
+#ifdef _FFR_POSTGRESQL_RECONNECT_HACK
+			if (status >= 0)
+			{
+				const char *estr;
+
+				estr = odbx_error(odbx, db->db_status);
+
+				if (estr != NULL &&
+				    strncmp(estr, "FATAL:", 6) == 0)
+					status = -1;
+			}
+#endif /* _FFR_POSTGRESQL_RECONNECT_HACK */
+
+			if (status < 0)
+			{
+				(void) odbx_unbind(odbx);
+				(void) odbx_finish(odbx);
+
+# ifdef _FFR_DB_HANDLE_POOLS
+				dkimf_db_hp_dead((struct handle_pool *) db->db_handle);
+# else /* _FFR_DB_HANDLE_POOLS */
+				db->db_iflags |= DKIMF_DB_IFLAG_RECONNECT;
+# endif /* _FFR_DB_HANDLE_POOLS */
+
+				if (db->db_lock != NULL)
+					(void) pthread_mutex_unlock(db->db_lock);
+
+				return dkimf_db_get(db, buf, buflen, req,
+				                    reqnum, exists);
+			}
+			else
+			{
+				if (db->db_lock != NULL)
+					(void) pthread_mutex_unlock(db->db_lock);
+				return err;
+			}
 		}
 
 		for (rescnt = 0; ; rescnt++)
 		{
-			err = odbx_result((odbx_t *) db->db_handle,
-			                  &result, NULL, 0);
+			err = odbx_result(odbx, &result, NULL, 0);
 			if (err < 0)
 			{
+				int status;
 				db->db_status = err;
+
+				if (reconnected)
+				{
+					if (db->db_lock != NULL)
+						(void) pthread_mutex_unlock(db->db_lock);
+# ifdef _FFR_DB_HANDLE_POOLS
+					dkimf_db_hp_put((struct handle_pool *) db->db_handle,
+					                (void *) odbx);
+# endif /* _FFR_DB_HANDLE_POOLS */
+					return err;
+				}
+
+				status = odbx_error_type(odbx, err);
+
+#ifdef _FFR_POSTGRESQL_RECONNECT_HACK
+				if (status >= 0)
+				{
+					const char *estr;
+
+					estr = odbx_error(odbx, db->db_status);
+
+					if (estr != NULL &&
+					    strncmp(estr, "FATAL:", 6) == 0)
+						status = -1;
+				}
+#endif /* _FFR_POSTGRESQL_RECONNECT_HACK */
+
+				if (status < 0)
+				{
+					(void) odbx_unbind(odbx);
+					(void) odbx_finish(odbx);
+
+# ifdef _FFR_DB_HANDLE_POOLS
+					dkimf_db_hp_dead((struct handle_pool *) db->db_handle);
+# else /* _FFR_DB_HANDLE_POOLS */
+					db->db_iflags |= DKIMF_DB_IFLAG_RECONNECT;
+# endif /* _FFR_DB_HANDLE_POOLS */
+
+					if (db->db_lock != NULL)
+						(void) pthread_mutex_unlock(db->db_lock);
+
+					return dkimf_db_get(db, buf, buflen, req,
+					                    reqnum, exists);
+				}
+
+				if (db->db_lock != NULL)
+					(void) pthread_mutex_unlock(db->db_lock);
+# ifdef _FFR_DB_HANDLE_POOLS
+				dkimf_db_hp_put((struct handle_pool *) db->db_handle,
+				                (void *) odbx);
+# endif /* _FFR_DB_HANDLE_POOLS */
+
 				return err;
 			}
 			else if (err == ODBX_RES_DONE)
@@ -2567,6 +3101,13 @@ dkimf_db_get(DKIMF_DB db, void *buf, size_t buflen,
 				if (exists != NULL && rescnt == 0)
 					*exists = FALSE;
 				err = odbx_result_finish(result);
+				if (db->db_lock != NULL)
+					(void) pthread_mutex_unlock(db->db_lock);
+# ifdef _FFR_DB_HANDLE_POOLS
+				dkimf_db_hp_put((struct handle_pool *) db->db_handle,
+				                (void *) odbx);
+# endif /* _FFR_DB_HANDLE_POOLS */
+
 				return 0;
 			}
 
@@ -2577,6 +3118,12 @@ dkimf_db_get(DKIMF_DB db, void *buf, size_t buflen,
 				{
 					db->db_status = err;
 					err = odbx_result_finish(result);
+					if (db->db_lock != NULL)
+						(void) pthread_mutex_unlock(db->db_lock);
+# ifdef _FFR_DB_HANDLE_POOLS
+					dkimf_db_hp_put((struct handle_pool *) db->db_handle,
+					                (void *) odbx);
+# endif /* _FFR_DB_HANDLE_POOLS */
 					return db->db_status;
 				}
 				else if (err == ODBX_RES_DONE)
@@ -2633,6 +3180,14 @@ dkimf_db_get(DKIMF_DB db, void *buf, size_t buflen,
 			err = odbx_result_finish(result);
 		}
 
+		if (db->db_lock != NULL)
+			(void) pthread_mutex_unlock(db->db_lock);
+
+# ifdef _FFR_DB_HANDLE_POOLS
+		dkimf_db_hp_put((struct handle_pool *) db->db_handle,
+		                (void *) odbx);
+# endif /* _FFR_DB_HANDLE_POOLS */
+
 		return 0;
 	  }
 #endif /* USE_ODBX */
@@ -2958,8 +3513,10 @@ dkimf_db_get(DKIMF_DB db, void *buf, size_t buflen,
 
 		lua = (struct dkimf_db_lua *) db->db_data;
 
-		status = dkimf_lua_db_hook(lua->lua_script, (const char *) buf,
-		                           &lres);
+		status = dkimf_lua_db_hook((const char *) lua->lua_script,
+		                           lua->lua_scriptlen,
+		                           (const char *) buf, &lres,
+		                           NULL, NULL);
 		if (status != 0)
 			return -1;
 
@@ -3023,13 +3580,20 @@ dkimf_db_close(DKIMF_DB db)
 
 		if ((db->db_iflags & DKIMF_DB_IFLAG_FREEARRAY) != 0)
 		{
-			for (c = 0; db->db_array != NULL; c++)
+			for (c = 0; db->db_array[c] != NULL; c++)
 				free(db->db_array[c]);
 		}
 		free(db->db_array);
 		db->db_array = NULL;
 	}
 
+	if (db->db_lock != NULL &&
+	    (db->db_flags & DKIMF_DB_FLAG_MAKELOCK) != 0)
+	{
+		pthread_mutex_destroy(db->db_lock);
+		free(db->db_lock);
+	}
+
 	switch (db->db_type)
 	{
 	  case DKIMF_DB_TYPE_FILE:
@@ -3066,8 +3630,13 @@ dkimf_db_close(DKIMF_DB db)
 
 #ifdef USE_ODBX
 	  case DKIMF_DB_TYPE_DSN:
+# ifdef _FFR_DB_HANDLE_POOLS
+		dkimf_db_hp_free((struct handle_pool *) db->db_handle);
+# else /* _FFR_DB_HANDLE_POOLS */
 		(void) odbx_finish((odbx_t *) db->db_handle);
+# endif /* _FFR_DB_HANDLE_POOLS */
 		free(db->db_data);
+		free(db);
 		return 0;
 #endif /* USE_ODBX */
 
@@ -3369,7 +3938,6 @@ dkimf_db_walk(DKIMF_DB db, _Bool first, void *key, size_t *keylen,
 		int fields;
 		odbx_result_t *result;
 		struct dkimf_db_dsn *dsn;
-		char query[BUFRSZ];
 
 		dsn = (struct dkimf_db_dsn *) db->db_data;
 		result = (odbx_result_t *) db->db_cursor;
@@ -3622,20 +4190,172 @@ dkimf_db_walk(DKIMF_DB db, _Bool first, void *key, size_t *keylen,
 	}
 }
 
+/*
+**  DKIMF_DB_MKARRAY_BASE -- make a (char *) array treating the DB as a
+**                           delta to a provided base
+**
+**  Parameters:
+**  	db -- a DKIMF_DB handle
+**  	a -- array (returned)
+**  	base -- base array
+** 
+**  Return value:
+**  	Length of the created array, or -1 on error/empty.
+*/
+
+static int
+dkimf_db_mkarray_base(DKIMF_DB db, char ***a, const char **base)
+{
+	_Bool found;
+	int c;
+	int status;
+	int nalloc = 0;
+	int nout = 0;
+	int nbase;
+	size_t buflen;
+	char **out = NULL;
+	char buf[BUFRSZ + 1];
+
+	assert(db != NULL);
+	assert(a != NULL);
+
+	/* count base elements */
+	for (nbase = 0; base[nbase] != NULL; nbase++)
+		continue;
+
+	/* initialize output array */
+	nalloc = MAX(nbase, 16);
+	out = (char **) malloc(sizeof(char *) * nalloc);
+	if (out == NULL)
+		return -1;
+	out[0] = NULL;
+
+	/* copy the base array modulo removals in the DB */
+	for (c = 0; c < nbase; c++)
+	{
+		memset(buf, '\0', sizeof buf);
+
+		snprintf(buf, sizeof buf, "-%s", base[c]);
+
+		found = FALSE;
+		status = dkimf_db_get(db, buf, 0, NULL, 0, &found);
+		if (status != 0)
+		{
+			for (c = 0; c < nout; c++)
+				free(out[c]);
+			free(out);
+			return -1;
+		}
+
+		if (!found)
+		{
+			if (nout == nalloc - 1)
+			{
+				char **new;
+
+				new = (char **) realloc(out,
+				                        sizeof(char *) * (nalloc * 2));
+				if (new == NULL)
+				{
+					for (c = 0; c < nout; c++)
+						free(out[c]);
+					free(out);
+					return -1;
+				}
+
+				out = new;
+				nalloc *= 2;
+			}
+
+			out[nout] = strdup(base[c]);
+			if (out[nout] == NULL)
+			{
+				for (c = 0; c < nout; c++)
+					free(out[c]);
+				free(out);
+				return -1;
+			}
+
+			nout++;
+			out[nout] = NULL;
+		}
+	}
+
+	/* now add any in the DB that aren't in the array */
+	for (c = 0; ; c++)
+	{
+		buflen = sizeof buf - 1;
+		memset(buf, '\0', sizeof buf);
+
+		status = dkimf_db_walk(db, (c == 0), buf, &buflen, NULL, 0);
+		if (status == -1)
+		{
+			for (c = 0; c < nout; c++)
+				free(out[c]);
+			free(out);
+			return -1;
+		}
+		else if (status == 1)
+		{
+			break;
+		}
+		else if (buf[0] != '+')
+		{
+			continue;
+		}
+
+		if (nout == nalloc - 1)
+		{
+			char **new;
+
+			new = (char **) realloc(out,
+			                        sizeof(char *) * (nalloc * 2));
+			if (new == NULL)
+			{
+				for (c = 0; c < nout; c++)
+					free(out[c]);
+				free(out);
+				return -1;
+			}
+
+			out = new;
+			nalloc *= 2;
+		}
+
+		out[nout] = strdup(&buf[1]);
+		if (out[nout] == NULL)
+		{
+			for (c = 0; c < nout; c++)
+				free(out[c]);
+			free(out);
+			return -1;
+		}
+
+		nout++;
+		out[nout] = NULL;
+	}
+
+	*a = out;
+	return nout;
+}
+
 /*
 **  DKIMF_DB_MKARRAY -- make a (char *) array of DB contents
 **
 **  Parameters:
 **  	db -- a DKIMF_DB handle
 **  	a -- array (returned)
+**  	base -- base array (may be NULL)
 **
 **  Return value:
 **  	Length of the created array, or -1 on error/empty.
 */
 
 int
-dkimf_db_mkarray(DKIMF_DB db, char ***a)
+dkimf_db_mkarray(DKIMF_DB db, char ***a, const char **base)
 {
+	_Bool found;
+	int status;
 	char **out;
 
 	assert(db != NULL);
@@ -3658,6 +4378,13 @@ dkimf_db_mkarray(DKIMF_DB db, char ***a)
 		return db->db_nrecs;
 	}
 
+	found = FALSE;
+	status = dkimf_db_get(db, "*", 0, NULL, 0, &found);
+	if (status != 0)
+		return -1;
+	if (found && base != NULL)
+		return dkimf_db_mkarray_base(db, a, base);
+
 	switch (db->db_type)
 	{
 	  case DKIMF_DB_TYPE_FILE:
@@ -3863,6 +4590,122 @@ dkimf_db_rewalk(DKIMF_DB db, char *str, DKIMF_DBDATA req, unsigned int reqnum,
 	return 1;
 }
 
+#ifdef _FFR_OVERSIGN
+/*
+**  DKIMF_DB_MERGE -- append the contents of one DB into another
+**
+**  Parameters:
+**  	dst -- destination DB
+**  	src -- source DB
+**
+**  Return value:
+**  	Number of new nodes added to "dst"; -1 on error.
+*/
+
+int
+dkimf_db_merge(DKIMF_DB dst, DKIMF_DB src)
+{
+	_Bool found;
+	int status;
+	int merged = 0;
+	struct dkimf_db_list *cur;
+
+	assert(dst != NULL);
+	assert(src != NULL);
+
+	if ((dst->db_type != DKIMF_DB_TYPE_CSL &&
+	     dst->db_type != DKIMF_DB_TYPE_FILE) ||
+	    (src->db_type != DKIMF_DB_TYPE_CSL &&
+	     src->db_type != DKIMF_DB_TYPE_FILE))
+		return -1;
+
+	for (cur = (struct dkimf_db_list *) src->db_handle;
+	     cur != NULL;
+	     cur = cur->db_list_next)
+	{
+		found = FALSE;
+
+		status = dkimf_db_get(dst, cur->db_list_value, 0, NULL, 0,
+		                      &found);
+
+		if (status != 0)
+			return -1;
+
+		if (!found)
+		{
+			struct dkimf_db_list *new;
+
+			new = (struct dkimf_db_list *) malloc(sizeof *new);
+			if (new == NULL)
+				return -1;
+
+			new->db_list_next = (struct dkimf_db_list *) dst->db_handle;
+
+			new->db_list_key = strdup(cur->db_list_key);
+			if (new->db_list_key == NULL)
+			{
+				free(new);
+				return -1;
+			}
+
+			if (cur->db_list_value == NULL)
+			{
+				new->db_list_value = NULL;
+			}
+			else
+			{
+				new->db_list_value = strdup(cur->db_list_value);
+				if (new->db_list_value == NULL)
+				{
+					free(new->db_list_key);
+					free(new);
+					return -1;
+				}
+			}
+
+			dst->db_handle = (struct dkimf_db_list *) dst->db_handle;
+
+			merged++;
+		}
+	}
+
+	return merged;
+}
+#endif /* _FFR_OVERSIGN */
+
+/*
+**  DKIMF_DB_FD -- retrieve a file descriptor associated with a database
+**
+**  Parameters:
+**  	db -- DKIMF_DB handle
+**
+**  Return value:
+**  	File descriptor associated with the DB, or -1 if none.
+*/
+
+int
+dkimf_db_fd(DKIMF_DB db)
+{
+	int fd = -1;
+
+#ifdef USE_DB
+	if (db->db_type == DKIMF_DB_TYPE_BDB)
+	{
+		DB *bdb;
+
+		bdb = (DB *) db->db_handle;
+
+# if DB_VERSION_CHECK(2,0,0)
+		(void) bdb->fd(bdb, &fd);
+# else /* DB_VERSION_CHECK(2,0,0) */
+		fd = bdb->fd(bdb);
+# endif /* DB_VERSION_CHECK(2,0,0) */
+	}
+#endif /* USE_DB */
+
+	return fd;
+}
+
 /*
 **  DKIMF_DB_SET_LDAP_PARAM -- set an LDAP parameter
 **
