@@ -2,16 +2,16 @@
 **  Copyright (c) 2008 Sendmail, Inc. and its suppliers.
 **      All rights reserved.
 **
-**  Copyright (c) 2009, 2010, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2009-2011, The OpenDKIM Project.  All rights reserved.
 **
-**  $Id: opendkim-db.h,v 1.11.2.2 2010/06/28 18:46:42 cm-msk Exp $
+**  $Id: opendkim-db.h,v 1.15 2010/09/16 04:47:39 cm-msk Exp $
 */
 
 #ifndef _OPENDKIM_DB_H_
 #define _OPENDKIM_DB_H_
 
 #ifndef lint
-static char opendkim_db_h_id[] = "@(#)$Id: opendkim-db.h,v 1.11.2.2 2010/06/28 18:46:42 cm-msk Exp $";
+static char opendkim_db_h_id[] = "@(#)$Id: opendkim-db.h,v 1.15 2010/09/16 04:47:39 cm-msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -24,6 +24,7 @@ static char opendkim_db_h_id[] = "@(#)$Id: opendkim-db.h,v 1.11.2.2 2010/06/28 1
 #define	DKIMF_DB_FLAG_MATCHBOTH	0x04
 #define	DKIMF_DB_FLAG_VALLIST	0x08
 #define	DKIMF_DB_FLAG_USETLS	0x10
+#define	DKIMF_DB_FLAG_MAKELOCK	0x20
 
 #define	DKIMF_DB_TYPE_UNKNOWN	(-1)
 #define	DKIMF_DB_TYPE_FILE	0
@@ -67,13 +68,18 @@ struct dkimf_db_data
 typedef struct dkimf_db_data * DKIMF_DBDATA;
 
 #define	DKIMF_DB_DATA_BINARY	0x01		/* data is binary */
+#define	DKIMF_DB_DATA_OPTIONAL	0x02		/* data is optional */
 
 /* prototypes */
 extern int dkimf_db_close __P((DKIMF_DB));
 extern int dkimf_db_delete __P((DKIMF_DB, void *, size_t));
+extern int dkimf_db_fd __P((DKIMF_DB));
 extern int dkimf_db_get __P((DKIMF_DB, void *, size_t,
                              DKIMF_DBDATA, unsigned int, _Bool *));
-extern int dkimf_db_mkarray __P((DKIMF_DB, char ***));
+extern int dkimf_db_mkarray __P((DKIMF_DB, char ***, const char **));
+#ifdef _FFR_OVERSIGN
+extern int dkimf_db_merge __P((DKIMF_DB, DKIMF_DB));
+#endif /* _FFR_OVERSIGN */
 extern int dkimf_db_open __P((DKIMF_DB *, char *, u_int flags,
                               pthread_mutex_t *, char **));
 extern int dkimf_db_put __P((DKIMF_DB, void *, size_t, void *, size_t));
