@@ -2,13 +2,13 @@
 **  Copyright (c) 2005-2009 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  Copyright (c) 2009, 2010, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2009-2011, The OpenDKIM Project.  All rights reserved.
 **
-**  $Id: util.c,v 1.35.2.3 2010/07/13 18:28:54 cm-msk Exp $
+**  $Id: util.c,v 1.47.2.1 2010/10/27 21:43:09 cm-msk Exp $
 */
 
 #ifndef lint
-static char util_c_id[] = "@(#)$Id: util.c,v 1.35.2.3 2010/07/13 18:28:54 cm-msk Exp $";
+static char util_c_id[] = "@(#)$Id: util.c,v 1.47.2.1 2010/10/27 21:43:09 cm-msk Exp $";
 #endif /* !lint */
 
 #include "build-config.h"
@@ -16,6 +16,7 @@ static char util_c_id[] = "@(#)$Id: util.c,v 1.35.2.3 2010/07/13 18:28:54 cm-msk
 /* system includes */
 #include <sys/param.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <sys/file.h>
@@ -37,6 +38,13 @@ static char util_c_id[] = "@(#)$Id: util.c,v 1.35.2.3 2010/07/13 18:28:54 cm-msk
 # include <regex.h>
 #endif /* _FFR_REPLACE_RULES */
 
+#ifdef HAVE_PATHS_H
+# include <paths.h>
+#endif /* HAVE_PATHS_H */
+#ifndef _PATH_DEVNULL
+# define _PATH_DEVNULL		"/dev/null"
+#endif /* ! _PATH_DEVNULL */
+
 #ifdef SOLARIS
 # if SOLARIS <= 20600
 #  define socklen_t size_t
@@ -51,6 +59,9 @@ static char util_c_id[] = "@(#)$Id: util.c,v 1.35.2.3 2010/07/13 18:28:54 cm-msk
 #include "util.h"
 #include "opendkim-db.h"
 
+/* macros */
+#define	DEFARGS		8
+
 /* missing definitions */
 #ifndef INADDR_NONE
 # define INADDR_NONE	((uint32_t) -1)
@@ -67,6 +78,10 @@ static char *optlist[] =
 	"DEBUG",
 #endif /* DEBUG */
 
+#if POLL
+	"POLL",
+#endif /* POLL */
+
 #if POPAUTH
 	"POPAUTH",
 #endif /* POPAUTH */
@@ -99,13 +114,21 @@ static char *optlist[] =
 	"VERIFY_DOMAINKEYS",
 #endif /* VERIFY_DOMAINKEYS */
 
+#ifdef _FFR_ADSP_LISTS
+	"_FFR_ADSP_LISTS",
+#endif /* _FFR_ADSP_LISTS */
+
+#ifdef _FFR_ATPS
+	"_FFR_ATPS",
+#endif /* _FFR_ATPS */
+
 #ifdef _FFR_BODYLENGTH_DB
 	"_FFR_BODYLENGTH_DB",
 #endif /* _FFR_BODYLENGTH_DB */
 
-#if _FFR_CAPTURE_UNKNOWN_ERRORS
-	"_FFR_CAPTURE_UNKNOWN_ERRORS",
-#endif /* _FFR_CAPTURE_UNKNOWN_ERRORS */
+#ifdef _FFR_DEFAULT_SENDER
+	"_FFR_DEFAULT_SENDER",
+#endif /* _FFR_DEFAULT_SENDER */
 
 #if _FFR_DIFFHEADERS
 	"_FFR_DIFFHEADERS",
@@ -115,22 +138,34 @@ static char *optlist[] =
 	"_FFR_DKIM_REPUTATION",
 #endif /* _FFR_DKIM_REPUTATION */
 
-#if _FFR_DNSUPGRADE
-	"_FFR_DNSUPGRADE",
-#endif /* _FFR_DNSUPGRADE */
-
 #if _FFR_IDENTITY_HEADER
 	"_FFR_IDENTITY_HEADER",
 #endif /* _FFR_IDENTITY_HEADER */
 
 #if _FFR_LDAP_CACHING
 	"_FFR_LDAP_CACHING",
-#endif /* _FFR_IDENTITY_HEADER */
+#endif /* _FFR_LDAP_CACHING */
+
+#if _FFR_MAXVERIFY
+	"_FFR_MAXVERIFY",
+#endif /* _FFR_MAXVERIFY */
+
+#if _FFR_OVERSIGN
+	"_FFR_OVERSIGN",
+#endif /* _FFR_OVERSIGN */
 
 #if _FFR_PARSE_TIME
 	"_FFR_PARSE_TIME",
 #endif /* _FFR_PARSE_TIME */
 
+#if _FFR_POSTGRESQL_RECONNECT_HACK
+	"_FFR_POSTGRESQL_RECONNECT_HACK",
+#endif /* _FFR_POSTGRESQL_RECONNECT_HACK */
+
+#if _FFR_RBL
+	"_FFR_RBL",
+#endif /* _FFR_RBL */
+
 #if _FFR_REDIRECT
 	"_FFR_REDIRECT",
 #endif /* _FFR_REDIRECT */
@@ -163,14 +198,14 @@ static char *optlist[] =
 	"_FFR_STATS",
 #endif /* _FFR_STATS */
 
+#if _FFR_STATSEXT
+	"_FFR_STATSEXT",
+#endif /* _FFR_STATSEXT */
+
 #if _FFR_VBR
 	"_FFR_VBR",
 #endif /* _FFR_VBR */
 
-#if _FFR_ZTAGS
-	"_FFR_ZTAGS",
-#endif /* _FFR_ZTAGS */
-
 	NULL
 };
 
@@ -180,7 +215,7 @@ struct dkimf_dstring
 	int			ds_alloc;
 	int			ds_max;
 	int			ds_len;
-	char *			ds_buf;
+	u_char *		ds_buf;
 };
 
 /* base64 alphabet */
@@ -196,7 +231,7 @@ static unsigned char alphabet[64] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopq
 **  	TRUE if "str" is either zero-length or contains only whitespace
 */
 
-bool
+_Bool
 dkimf_isblank(char *str)
 {
 	char *p;
@@ -225,7 +260,7 @@ dkimf_isblank(char *str)
 void
 dkimf_optlist(FILE *where)
 {
-	bool first = TRUE;
+	_Bool first = TRUE;
 	int c;
 
 	assert(where != NULL);
@@ -240,7 +275,7 @@ dkimf_optlist(FILE *where)
 
 		fprintf(where, "\t\t%s\n", optlist[c]);
 	}
-        fprintf(where, "%s\n", LIBOPENDKIM_FEATURE_STRING);
+        fprintf(where, "\t%s\n", LIBOPENDKIM_FEATURE_STRING);
 }
 
 /*
@@ -342,10 +377,10 @@ dkimf_lowercase(u_char *str)
 **  	TRUE if there's a match, FALSE otherwise.
 */
 
-bool
+_Bool
 dkimf_checkhost(DKIMF_DB db, char *host)
 {
-	bool exists;
+	_Bool exists;
 	int status;
 	char *p;
 	char buf[BUFRSZ + 1];
@@ -394,10 +429,10 @@ dkimf_checkhost(DKIMF_DB db, char *host)
 **  	TRUE if there's a match, FALSE otherwise.
 */
 
-bool
+_Bool
 dkimf_checkip(DKIMF_DB db, struct sockaddr *ip)
 {
-	bool exists;
+	_Bool exists;
 	char ipbuf[DKIM_MAXHOSTNAMELEN + 1];
 
 	assert(ip != NULL);
@@ -427,7 +462,7 @@ dkimf_checkip(DKIMF_DB db, struct sockaddr *ip)
 		dst_len = sizeof ipbuf - 1;
 
 		inet_ntop(AF_INET6, &addr, dst, dst_len);
-		dkimf_lowercase(dst);
+		dkimf_lowercase((u_char *) dst);
 
 		exists = FALSE;
 
@@ -457,7 +492,7 @@ dkimf_checkip(DKIMF_DB db, struct sockaddr *ip)
 			dst_len = sizeof ipbuf - 1;
 
 			inet_ntop(AF_INET6, &addr, dst, dst_len);
-			dkimf_lowercase(dst);
+			dkimf_lowercase((u_char *) dst);
 
 			sz = strlcat(ipbuf, "/", sizeof ipbuf);
 			if (sz >= sizeof ipbuf)
@@ -501,7 +536,7 @@ dkimf_checkip(DKIMF_DB db, struct sockaddr *ip)
 
 	if (ip->sa_family == AF_INET)
 	{
-		bool exists;
+		_Bool exists;
 		int c;
 		int status;
 		int bits;
@@ -624,10 +659,10 @@ dkimf_initpopauth(void)
 **  	- does the key contain anything meaningful, like an expiry time?
 */
 
-bool
+_Bool
 dkimf_checkpopauth(DKIMF_DB db, struct sockaddr *ip)
 {
-	bool exists;
+	_Bool exists;
 	int status;
 	struct sockaddr_in *sin;
 	struct in_addr addr;
@@ -669,7 +704,7 @@ dkimf_checkpopauth(DKIMF_DB db, struct sockaddr *ip)
 **  	Prints an error message when appropriate.
 */
 
-bool
+_Bool
 dkimf_load_replist(FILE *in, struct replace **list)
 {
 	int line;
@@ -805,8 +840,10 @@ dkimf_inet_ntoa(struct in_addr a, char *buf, size_t buflen)
 void
 dkimf_trimspaces(u_char *str)
 {
+	size_t len = 0;
 	u_char *p;
 	u_char *last;
+	u_char *firsttext = NULL;
 
 	assert(str != NULL);
 
@@ -814,18 +851,29 @@ dkimf_trimspaces(u_char *str)
 
 	for (p = str; *p != '\0'; p++)
 	{
-		if (isascii(*p) && isspace(*p) && last == NULL)
-		{
-			last = p;
-			continue;
-		}
+		len++;
 
-		if (!isascii(*p) || !isspace(*p))
+		if (isascii(*p) && isspace(*p))
+		{
+			if (last == NULL)
+			{
+				last = p;
+				continue;
+			}
+		}
+		else
+		{
 			last = NULL;
+			if (firsttext == NULL)
+				firsttext = p;
+		}
 	}
 
 	if (last != NULL)
 		*last = '\0';
+
+	if (firsttext != NULL && firsttext != str)
+		memmove(str, firsttext, len - (firsttext - str) + 1);
 }
 
 /*
@@ -911,7 +959,7 @@ dkimf_mkpath(char *path, size_t pathlen, char *root, char *file)
 **  	found in the list.
 */
 
-bool
+_Bool
 dkimf_hostlist(char *host, char **list)
 {
 	int c;
@@ -955,11 +1003,11 @@ dkimf_hostlist(char *host, char **list)
 **  	specific size.
 */
 
-static bool
+static _Bool
 dkimf_dstring_resize(struct dkimf_dstring *dstr, int len)
 {
 	int newsz;
-	char *new;
+	u_char *new;
 
 	assert(dstr != NULL);
 	assert(len > 0);
@@ -1082,15 +1130,15 @@ dkimf_dstring_free(struct dkimf_dstring *dstr)
 **  	The dstring may be resized.
 */
 
-bool
-dkimf_dstring_copy(struct dkimf_dstring *dstr, char *str)
+_Bool
+dkimf_dstring_copy(struct dkimf_dstring *dstr, u_char *str)
 {
 	int len;
 
 	assert(dstr != NULL);
 	assert(str != NULL);
 
-	len = strlen(str);
+	len = strlen((char *) str);
 
 	/* too big? */
 	if (dstr->ds_max > 0 && len >= dstr->ds_max)
@@ -1105,7 +1153,8 @@ dkimf_dstring_copy(struct dkimf_dstring *dstr, char *str)
 	}
 
 	/* copy */
-	dstr->ds_len = strlcpy(dstr->ds_buf, str, dstr->ds_alloc);
+	dstr->ds_len = strlcpy((char *) dstr->ds_buf, (char *) str,
+	                       dstr->ds_alloc);
 
 	return TRUE;
 }
@@ -1124,15 +1173,15 @@ dkimf_dstring_copy(struct dkimf_dstring *dstr, char *str)
 **  	The dstring may be resized.
 */
 
-bool
-dkimf_dstring_cat(struct dkimf_dstring *dstr, char *str)
+_Bool
+dkimf_dstring_cat(struct dkimf_dstring *dstr, u_char *str)
 {
 	int len;
 
 	assert(dstr != NULL);
 	assert(str != NULL);
 
-	len = strlen(str) + dstr->ds_len;
+	len = strlen((char *) str) + dstr->ds_len;
 
 	/* too big? */
 	if (dstr->ds_max > 0 && len >= dstr->ds_max)
@@ -1147,7 +1196,8 @@ dkimf_dstring_cat(struct dkimf_dstring *dstr, char *str)
 	}
 
 	/* append */
-	dstr->ds_len = strlcat(dstr->ds_buf, str, dstr->ds_alloc);
+	dstr->ds_len = strlcat((char *) dstr->ds_buf, (char *) str,
+	                       dstr->ds_alloc);
 
 	return TRUE;
 }
@@ -1166,7 +1216,7 @@ dkimf_dstring_cat(struct dkimf_dstring *dstr, char *str)
 **  	The dstring may be resized.
 */
 
-bool
+_Bool
 dkimf_dstring_cat1(struct dkimf_dstring *dstr, int c)
 {
 	int len;
@@ -1204,7 +1254,7 @@ dkimf_dstring_cat1(struct dkimf_dstring *dstr, int c)
 **  	Pointer to the NULL-terminated contents of "dstr".
 */
 
-char *
+u_char *
 dkimf_dstring_get(struct dkimf_dstring *dstr)
 {
 	assert(dstr != NULL);
@@ -1377,7 +1427,7 @@ dkimf_socket_cleanup(char *sockspec)
 **  	TRUE iff "dest" was big enough (based on destlen)
 */
 
-bool
+_Bool
 dkimf_mkregexp(char *src, char *dst, size_t dstlen)
 {
 	char *p;
@@ -1537,7 +1587,7 @@ dkimf_base64_encode_file(infd, out, lm, rm, initial)
 **  	TRUE iff d1 is a subdomain of d2.
 */
 
-bool
+_Bool
 dkimf_subdomain(char *d1, char *d2)
 {
 	char *p;
@@ -1607,3 +1657,84 @@ dkimf_ipstring(char *buf, size_t buflen, struct sockaddr_storage *ss)
 		break;
 	}
 }
+
+#ifdef USE_UNBOUND
+/*
+**  DKIMF_TIMESPEC_PAST -- return TRUE iff the time described by a timespec
+**                         structure has passed
+**
+**  Parameters:
+**  	ts -- timespec structure to evaluate
+**
+**  Return value:
+**  	TRUE if "tv" refers to a time in the past, false otherwise.
+*/
+
+_Bool
+dkimf_timespec_past(struct timespec *ts)
+{
+	struct timeval now;
+
+	assert(ts != NULL);
+
+	(void) gettimeofday(&now, NULL);
+
+	if (now.tv_sec > ts->tv_sec ||
+	    (now.tv_sec == ts->tv_sec && now.tv_usec * 1000 > ts->tv_nsec))
+		return TRUE;
+	else
+		return FALSE;
+}
+
+/*
+**  DKIMF_WAIT_FD -- wait for a descriptor to become read-ready
+**
+**  Parameters:
+**  	fd -- descriptor of interest
+**  	until -- maximum wait time
+**
+**  Return value:
+**  	1 -- descriptor is ready
+**  	0 -- timeout
+**  	-1 -- error
+*/
+
+int
+dkimf_wait_fd(int fd, struct timespec *until)
+{
+	fd_set fds;
+	struct timeval now;
+	struct timeval left;
+
+	assert(fd >= 0);
+
+	(void) gettimeofday(&now, NULL);
+
+	if (until != NULL)
+	{
+		if (until->tv_sec < now.tv_sec ||
+		    (until->tv_sec == now.tv_sec &&
+		     until->tv_nsec < now.tv_usec * 1000))
+		{
+			left.tv_sec = 0;
+			left.tv_usec = 0;
+		}
+		else
+		{
+			left.tv_sec = until->tv_sec - now.tv_sec;
+			left.tv_usec = until->tv_nsec / 1000 - now.tv_usec;
+
+			if (until->tv_nsec / 1000 < now.tv_usec)
+			{
+				left.tv_usec += 1000000;
+				left.tv_sec--;
+			}
+		}
+	}
+
+	FD_ZERO(&fds);
+	FD_SET(fd, &fds);
+
+	return select(fd + 1, &fds, NULL, NULL, until == NULL ? NULL : &left);
+}
+#endif /* USE_UNBOUND */
